#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import socket
import threading
import zipfile

from .filereceiver import FileReceiver


class FileReceiverQO100Multimedia(FileReceiver):
    def frame_status(self, chunk):
        return (int(chunk[1]) >> 4) & 0x03

    def frame_type(self, chunk):
        return int(chunk[1]) & 0x0f

    def chunk_sequence(self, chunk):
        return int(chunk[0]) | ((int(chunk[1]) >> 6) << 8)

    def chunk_offset(self, chunk):
        seq = self.chunk_sequence(chunk)
        if seq == 0:
            return 0
        return (seq - 1) * 219 + 164

    def chunk_data(self, chunk):
        seq = self.chunk_sequence(chunk)
        if seq == 0:
            return chunk[57:]
        data = chunk[2:]
        if not self.is_last_chunk(chunk):
            return data
        # Trim if we are in the last chunk
        length = (self._files[self._current_file].size
                  - self.chunk_offset(chunk)
                  if self._current_file is not None
                  else len(data))
        return data[:length]

    def file_size(self, chunk):
        if self.chunk_sequence(chunk) != 0:
            return None
        return (int(chunk[54]) << 16) | (int(chunk[55]) << 8) | int(chunk[56])

    def is_last_chunk(self, chunk):
        return self.frame_status(chunk) in (2, 3)

    def parse_chunk(self, chunk):
        if self.frame_type(chunk) >= 8:
            return None
        # Drop chunk if it's the last chunk and we do not have enough
        # information about the current file. This prevents self.chunk_data
        # from failing later on.
        if self.is_last_chunk(chunk):
            try:
                self.chunk_data(chunk)
            except Exception as e:
                print('Could not get file data for last chunk:', e)
                return None
        return chunk

    def file_id(self, chunk):
        if self.chunk_sequence(chunk) != 0:
            return None
        try:
            name = (str(chunk[2:52], encoding='ascii')
                    .rstrip('\x00')
                    .replace('\x00', ' '))
        except Exception as e:
            print('Could not obtain filename:', e)
            return None
        if self.frame_type(chunk) in [3, 4, 5]:
            # ASCII, HTML and binary files are zipped, so we add the .zip
            # extension
            name += '.zip'
        return name

    def on_completion(self, f):
        fname = f.path.name
        if not fname.endswith('.zip'):
            # Not a zip file. No need to unzip
            return
        f.f.flush()
        outname = fname[:-4]  # remove .zip
        try:
            data = zipfile.ZipFile(f.path).read(outname)
            with open(f.path.parent / outname, 'wb') as fdata:
                fdata.write(data)
                if fname.endswith('.blt.zip'):
                    threading.Thread(
                        target=send_to_websocket,
                        args=(data,)).run()
        except Exception as e:
            print('Could not unzip received file:', e)


# This is used in a new thread to send the file contents to the websocket
# server Python script (see examples/qo100-multimedia-beacon).
def send_to_websocket(data):
    port = 52002
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect(('127.0.0.1', port))
        s.send(data)
        s.close()
    except Exception as e:
        print('Could not send file to websocket server script:', e)


qo100_multimedia = FileReceiverQO100Multimedia



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017, 2018, 2019, 2020 Daniel Estevez <daniel@destevez.net>
# Copyright 2022 The Regents of the University of Colorado
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .ax25 import Header
from ..ccsds import space_packet as ccsds_space_packet
from ..adapters import PolynomialAdapter
from construct import Adapter, BitStruct, BitsInteger, Enum, Flag, Float32l, \
                      If, Int16sl, Int16ul, Int32sl, Int32ul, Int8sl, \
                      Int8ub, Int8ul, Padding, Struct, Switch


SecondaryHeader = Struct(
    'sh_coarse' / Int32ul,
    'sh_fine' / Int16ul,
)

cmd_opcodes = Enum(
    Int8ub,
    CMD_OPCODE_NOOP=0, CMD_OPCODE_ARM=1, CMD_OPCODE_RESET_STATS=2,
    CMD_OPCODE_FP_RESET=16, CMD_OPCODE_FP_RESET_RESULTS=17,
    CMD_OPCODE_FP_VALIDATE=18, CMD_OPCODE_FP_SET_TASK_STATE=19,
    CMD_OPCODE_FP_SET_WATCH_STATE=20, CMD_OPCODE_FP_DUMP_WATCH=21,
    CMD_OPCODE_FP_DUMP_RESULTS=22, CMD_OPCODE_FP_SET_WATCH_VALUES=23,
    CMD_OPCODE_FP_SET_WATCH_THRESH=24, CMD_OPCODE_FP_SET_WATCH_RESP=25,
    CMD_OPCODE_SET_PERF_MON_STATE=32, CMD_OPCODE_DES_RESET_STATS=33,
    CMD_OPCODE_DES_ADD_TASK=34, CMD_OPCODE_DES_SUB_TASK=35,
    CMD_OPCODE_DES_ADD_BACK=36, CMD_OPCODE_DES_SUB_BACK=37,
    CMD_OPCODE_DES_SET_TIME=38, CMD_OPCODE_SET_EVT_GROUP=48,
    CMD_OPCODE_SET_EVT_LOG=49, CMD_OPCODE_LOG_RESET_STATS=50,
    CMD_OPCODE_LOG_SET_MASK_STATE=51, CMD_OPCODE_LOG_RESET_LOG=52,
    CMD_OPCODE_LOG_ISSUE_LOG=53, CMD_OPCODE_LOAD_SEQUENCE=64,
    CMD_OPCODE_INIT_SEQUENCE=65, CMD_OPCODE_SET_SEQUENCE_STATE=66,
    CMD_OPCODE_VERIFY_SEQUENCE=67, CMD_OPCODE_STOP_SEQUENCE=68,
    CMD_OPCODE_FIND_SEQUENCE=69, CMD_OPCODE_INFO_SEQUENCE=70,
    CMD_OPCODE_DUMP_SEQUENCE=71, MEM_DUMP=80, MEM_LOAD=81, MEM_ERASE=82,
    MEM_XSUM=83, MEM_ABORT=84, MEM_RESET=85, MEM_LOAD_DWORD=86,
    MEM_LOAD_WORD=87, MEM_LOAD_BYTE=88, CMD_OPCODE_ISSUE_TLM_PKT=96,
    CMD_OPCODE_SET_TLM_RATE=97, CMD_OPCODE_SET_STREAM_STATE=98,
    CMD_OPCODE_PKT_SET_PRIORITY=99, CMD_OPCODE_UHF_PASS=128,
    CMD_OPCODE_UHF_INIT=129, CMD_OPCODE_UHF_RESP_STATE=130,
    CMD_OPCODE_UHF_SET_TX_GAP=131, CMD_OPCODE_ADCS_PASS=144,
    CMD_OPCODE_ADCS_RESET=145, CMD_OPCODE_ADCS_READ=146,
    CMD_OPCODE_ADCS_COARSE_POINT=147, CMD_OPCODE_ADCS_FINE_POINT=148,
    CMD_OPCODE_ADCS_RAM_POINT=149, CMD_OPCODE_ADCS_FINE_UPDATE=150,
    CMD_OPCODE_ADCS_RAM_UPDATE=151, CMD_OPCODE_ADCS_ECLIPSE_CHECK=152,
    CMD_OPCODE_EPS_PWR_ON=160, CMD_OPCODE_EPS_PWR_OFF=161,
    CMD_OPCODE_EPS_PWR_CYCLE=162, CMD_OPCODE_EPS_DEPLOY=163,
    CMD_OPCODE_EPS_HEATER_SETPOINT=164, CMD_OPCODE_EPS_HEATER_THRESH=165,
    CMD_OPCODE_EPS_ECLIPSE_THRESH=166, CMD_OPCODE_SD_WRITE_STATE=176,
    CMD_OPCODE_SD_READ=177, CMD_OPCODE_SD_READ_HALT=178,
    CMD_OPCODE_SD_PLAYBACK=179, CMD_OPCODE_SD_SELECT=181,
    CMD_OPCODE_SD_INIT=182, CMD_OPCODE_SD_SET_PARTITION=183,
    CMD_OPCODE_SD_UPDATE_PARTITION=184, CMD_OPCODE_SD_PWR_OFF=185,
    CMD_OPCODE_SD_RESET_FDRI=186, CMD_OPCODE_SD_SET_FDRI_LIMT=187,
    CMD_OPCODE_SD_TABLE_DEFAULT=188, CMD_OPCODE_CIP_SCI_MODE=208,
    CMD_OPCODE_SBAND_SET_PA=226, CMD_OPCODE_SBAND_SET_MODE=227,
    CMD_OPCODE_SBAND_SET_PA_LEVEL=228, CMD_OPCODE_SBAND_SET_SYNTH=229,
    CMD_OPCODE_SBAND_RESET=230, CMD_OPCODE_SBAND_DEBUG=231,
    CMD_OPCODE_SBAND_SET_ENCODER=232, CMD_OPCODE_SBAND_SYNC_ON=233,
    CMD_OPCODE_SBAND_SYNC_OFF=234, CMD_OPCODE_MODE_SET=240,
    CMD_OPCODE_MODE_AVOID=241, CMD_OPCODE_MODE_SET_THRESHOLD=242,
    CMD_OPCODE_MODE_CLEAR_CLT=243, CMD_OPCODE_MODE_SET_CLT_THRESHOLD=244,
    CMD_OPCODE_MODE_SET_LAUNCH_DELAY=245,
    CMD_OPCODE_MODE_SET_LAUNCH_DELAY_STATE=246,
    CMD_OPCODE_MODE_SET_DEPLOY_INT=247, CMD_OPCODE_MODE_SET_DEPLOY_FLAG=248,
    CMD_OPCODE_MODE_RESET_SC=249, CMD_OPCODE_MODE_AVOID_CLEAR=250,
    CMD_OPCODE_MODE_ECLIPSE_METHOD=251, CMD_OPCODE_MODE_AUTO_DIS=252,
    CMD_OPCODE_MODE_AUTO_ENA=253
)

pwr_status = BitStruct(
    'eclipse_state' / Enum(BitsInteger(1), SUN=0, ECLIPSE=1),
    'pwr_status_sd1' / Enum(BitsInteger(1), OFF=0, ON=1),
    'pwr_status_sd0' / Enum(BitsInteger(1), OFF=0, ON=1),
    'pwr_status_htr' / Enum(BitsInteger(1), OFF=0, ON=1),
    'pwr_status_sband' / Enum(BitsInteger(1), OFF=0, ON=1),
    'pwr_status_adcs' / Enum(BitsInteger(1), OFF=0, ON=1),
    'pwr_status_cip' / Enum(BitsInteger(1), OFF=0, ON=1),
    'pwr_status_daxss' / Enum(BitsInteger(1), OFF=0, ON=1)
)

alive_flags = BitStruct(
    'clt_state' / Enum(BitsInteger(1), OFF=0, ON=1),
    'alive_daxss' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'alive_cip' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'alive_adcs' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'alive_sband' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'alive_uhf' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'alive_sd1' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'alive_sd0' / Enum(BitsInteger(1), DEAD=0, ALIVE=1)
)

uhf_temp = BitStruct(
    'uhf_temp_buff' / BitsInteger(2),
    'uhf_temp' / BitsInteger(6)
)

uhf_config = BitStruct(
    'uhf_locked' / Enum(BitsInteger(1), OFF=0, ON=1),
    'uhf_readback' / Enum(BitsInteger(2), N=0, B=1, T=2),
    'uhf_swd' / Enum(BitsInteger(1), OFF=0, ON=1),
    'uhf_afc' / Enum(BitsInteger(1), OFF=0, ON=1),
    'uhf_echo' / Enum(BitsInteger(1), OFF=0, ON=1),
    'uhf_channel' / Enum(BitsInteger(2), A=0, B=1, D=2)
)

adcs_info = BitStruct(
    'adcs_att_valid' / BitsInteger(1),
    'adcs_refs_valid' / BitsInteger(1),
    'adcs_time_valid' / BitsInteger(1),
    'adcs_mode' / Enum(BitsInteger(1), SUN=0, FINE=1),
    'adcs_recom_sun_pt' / Enum(BitsInteger(1), NO=0, YES=1),
    'adcs_sun_pt_state' / Enum(BitsInteger(3), UNDEF_0=0, UNDEF_1=1,
                               SEARCH_INIT=2, SEARCHING=3, WAITING=4,
                               CONVERGING=5, ON_SUN=6, NOT_ACTIVE=7),
)

# Polynomial used for converting certain temperature telemetry points
# (obc_temp, eps_temp, int_temp, sp0_temp, sp1_temp, sp2_temp)
tempPoly = [91.394, -0.0894932, 3.55e-05, -6.26e-9, 1.89e-13]

# Polynomial used for converting bat0_temp and bat1_temp
batTempPoly = [-259.74, 1.471182747, 3.59e-4, 8.77e-8, 2.14e-11]

is1_beacon = Struct(
    'cmd_recv_count' / Int8ul,
    'cmd_fail_count' / Int8ul,
    'cmd_succ_count' / Int8ul,
    'cmd_succ_op' / cmd_opcodes,
    'cmd_fail_op' / cmd_opcodes,
    'cmd_fail_code' / Enum(Int8ul, SUCCESS=0, MODE=1, ARM=2,
                           SOURCE=3, OPCODE=4, METHOD=5, LENGTH=6,
                           RANGE=7),
    'pwr_status' / pwr_status,
    'sd_read_misc' / Int32ul,
    'sd_read_scic' / Int32ul,
    'sd_read_scid' / Int32ul,
    'sd_read_adcs' / Int32ul,
    'sd_read_beacon' / Int32ul,
    'sd_read_log' / Int32ul,
    'sd_write_misc' / Int32ul,
    'sd_write_scic' / Int32ul,
    'sd_write_scid' / Int32ul,
    'sd_write_adcs' / Int32ul,
    'sd_write_beacon' / Int32ul,
    'sd_write_log' / Int32ul,
    'cmd_loss_timer' / Int32ul,
    'alive_flags' / alive_flags,
    'cip_comstat' / Int32ul,
    'cip_temp1' / PolynomialAdapter([0.0, 0.007813], Int16sl),
    'cip_temp2' / PolynomialAdapter([0.0, 0.007813], Int16sl),
    'cip_temp3' / PolynomialAdapter([0.0, 0.007813], Int16sl),
    'uhf_temp' / uhf_temp,
    'uhf_config' / uhf_config,
    'sband_pa_curr' / PolynomialAdapter([0.0, 4e-05], Int16ul),
    'sband_pa_volt' / PolynomialAdapter([0.0, 0.004], Int16ul),
    'sband_rf_pwr' / PolynomialAdapter([0.0, 0.001139], Int16ul),
    'sband_pa_temp' / PolynomialAdapter([-50.0, 0.073242], Int16ul),
    'sband_top_temp' / PolynomialAdapter([0.0, 0.0625], Int16ul),
    'sband_bottom_temp' / PolynomialAdapter([0.0, 0.0625], Int16ul),
    'adcs_cmd_acpt' / Int8ul,
    'adcs_cmd_fail' / Int8ul,
    'adcs_time' / Int32ul,
    'adcs_info' / adcs_info,
    'adcs_star_temp' / PolynomialAdapter([0.0, 0.8], Int8sl),
    'adcs_wheel_temp1' / PolynomialAdapter([0.0, 0.005], Int16sl),
    'adcs_wheel_temp2' / PolynomialAdapter([0.0, 0.005], Int16sl),
    'adcs_wheel_temp3' / PolynomialAdapter([0.0, 0.005], Int16sl),
    'adcs_digi_bus_volt' / PolynomialAdapter([0.0, 0.00125], Int16sl),
    'adcs_sun_vec1' / PolynomialAdapter([0.0, 0.0001], Int16sl),
    'adcs_sun_vec2' / PolynomialAdapter([0.0, 0.0001], Int16sl),
    'adcs_sun_vec3' / PolynomialAdapter([0.0, 0.0001], Int16sl),
    'adcs_wheel_sp1' / PolynomialAdapter([0.0, 0.4], Int16sl),
    'adcs_wheel_sp2' / PolynomialAdapter([0.0, 0.4], Int16sl),
    'adcs_wheel_sp3' / PolynomialAdapter([0.0, 0.4], Int16sl),
    'adcs_body_rt1' / PolynomialAdapter([0.0, 5.0e-9], Int32sl),
    'adcs_body_rt2' / PolynomialAdapter([0.0, 5.0e-9], Int32sl),
    'adcs_body_rt3' / PolynomialAdapter([0.0, 5.0e-9], Int32sl),
    Padding(12),
    'daxss_time_sec' / Int32ul,
    'daxss_cmd_op' / Int8ul,
    'daxss_cmd_succ' / Int8ul,
    'daxss_cmd_fail' / Int8ul,
    'daxss_cdh_enables' / Int16ul,
    'daxss_cdh_temp' / PolynomialAdapter([0.0, 0.003906], Int16sl),
    'daxss_sps_rate' / Int32ul,
    'daxss_sps_x' / Int16ul,
    'daxss_sps_y' / Int16ul,
    'daxss_slow_count' / Int16ul,
    'bat_fg1' / PolynomialAdapter([0.0, 0.003906], Int16ul),
    'daxss_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'daxss_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'cdh_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'cdh_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'sband_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'sband_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'uhf_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'uhf_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'heater_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'heater_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'sp2_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'sp2_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'sp1_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'sp1_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'sp0_curr' / PolynomialAdapter([0.0, 0.00125], Int16ul),
    'sp0_volt' / PolynomialAdapter([0.0, 0.00125], Int16ul),
    'bat_vcell' / Int16ul,
    'gps_12v_2_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'gps_12v_2_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'gps_12v_1_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'gps_12v_1_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'bat_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'bat_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'adcs_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'adcs_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    '3p3_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    '3p3_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'cip_curr' / PolynomialAdapter([0.0, 0.0005], Int16ul),
    'cip_volt' / PolynomialAdapter([0.0, 0.001], Int16ul),
    'obc_temp' / PolynomialAdapter(tempPoly, Int16ul),
    'eps_temp' / PolynomialAdapter(tempPoly, Int16ul),
    'int_temp' / PolynomialAdapter(tempPoly, Int16ul),
    'sp0_temp' / PolynomialAdapter(tempPoly, Int16ul),
    'bat0_temp' / PolynomialAdapter(batTempPoly, Int16ul),
    'sp1_temp' / PolynomialAdapter(tempPoly, Int16ul),
    'bat1_temp' / PolynomialAdapter(batTempPoly, Int16ul),
    'sp2_temp' / PolynomialAdapter(tempPoly, Int16ul),
    'bat_fg3' / PolynomialAdapter([0.0, 0.003906], Int16ul),
    'bat0_temp_conv' / Float32l,
    'bat1_temp_conv' / Float32l,
    'mode' / Enum(Int8ul, PHOENIX=0, SAFE=1, SCID=2, SCIC=3)
)

inspiresat_1 = Struct(
    'ax25_header' / Header,
    'primary_header' / ccsds_space_packet.PrimaryHeader,
    'secondary_header' / If(
        lambda c: c.primary_header.secondary_header_flag,
        SecondaryHeader
    ),
    'packet' / Switch(
        lambda c: (c.primary_header.AP_ID),
        {
            (0x01): is1_beacon
        }
    )
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import asyncio
import functools
import logging
import websockets


async def ws_recv_alive(websocket):
    async for message in websocket:
        pass


async def ws_send_data(websocket, queue):
    while True:
        item = await queue.get()
        # For some reason we need to add 3 bytes before
        # the payload
        await websocket.send(b'\x00\x00\x00' + item)


async def tcp_client(queue):
    port = 52001
    logging.info(
        'connecting to GNU Radio flowgraph at TCP port %d', port)
    reader, writer = await asyncio.open_connection(
        '127.0.0.1', port)
    logging.info('connection to flowgraph established')
    while True:
        packet = await reader.read(221)
        packet_type = packet[1] & 0xf
        # packet_type == 8 corresponds to external data
        if packet_type != 8:
            continue
        payload = packet[2:]
        payload_type = payload[0]
        if payload_type not in [0, 1, 2, 3]:
            # unknown payload type
            continue
        if payload_type in [1, 3]:
            payload = payload[:1] + repack_8to6(payload[1:])
        await queue.put(payload)


async def handle_file(reader, writer, queue):
    max_len = 2**20
    data = await reader.read(max_len)
    logging.info('received new bulletin file')
    await queue.put(b'\x10' + data)


async def tcp_file_server(queue):
    port = 52002
    server = await asyncio.start_server(
        functools.partial(handle_file, queue=queue),
        '127.0.0.1', port)
    logging.info('starting TCP file server at port %d', port)
    async with server:
        await server.serve_forever()


def repack_8to6(data):
    """Repacks data from 8 bits/byte to 6 bits/byte"""
    out = bytearray()
    sby = 0
    sbi = 0
    dbi = 0
    byte = 0
    while True:
        bit = (data[sby] >> sbi) & 1
        byte |= bit << dbi
        sbi += 1
        if sbi == 8:
            sbi = 0
            sby += 1
            if sby == len(data):
                return bytes(out)
        dbi += 1
        if dbi == 6:
            dbi = 0
            out.append(byte)
            byte = 0


async def handle(websocket):
    logging.info('client connected')
    queue = asyncio.Queue()
    await asyncio.gather(
        tcp_client(queue),
        tcp_file_server(queue),
        ws_send_data(websocket, queue),
        ws_recv_alive(websocket))


async def main():
    logging.basicConfig(
        format='%(asctime)s:%(levelname)s:%(message)s',
        level=logging.INFO)
    logging.info('starting QO-100 multimedia beacon WebSocket server')
    async with websockets.serve(handle, 'localhost', 40134):
        await asyncio.Future()  # run forever


if __name__ == '__main__':
    asyncio.run(main())



# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: CCSDS descrambler
# Author: Daniel Estevez
# Description: CCSDS descrambler (input is unpacked, output is packed)
# GNU Radio version: 3.8.0.0

from gnuradio import blocks
from gnuradio import digital
from gnuradio import gr
from gnuradio.filter import firdes
from ..grpdu import pdu_to_tagged_stream, tagged_stream_to_pdu
from ..grtypes import byte_t


class ccsds_descrambler(gr.hier_block2):
    def __init__(self):
        gr.hier_block2.__init__(
            self,
            'CCSDS descrambler',
            gr.io_signature(0, 0, 0),
            gr.io_signature(0, 0, 0),
        )
        self.message_port_register_hier_in("in")
        self.message_port_register_hier_out("out")

        ##################################################
        # Blocks
        ##################################################
        self.digital_additive_scrambler_bb_0_0 = digital.additive_scrambler_bb(
            0xA9, 0xFF, 7, count=0, bits_per_byte=1,
            reset_tag_key="packet_len")
        self.blocks_unpacked_to_packed_xx_0_0_0_0 = (
            blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST))
        self.blocks_tagged_stream_to_pdu_0_0_0_0_0 = (
            tagged_stream_to_pdu(byte_t, 'packet_len'))
        self.blocks_tagged_stream_multiply_length_0_0_0_0 = (
            blocks.tagged_stream_multiply_length(
                gr.sizeof_char*1, 'packet_len', 1/8.0))
        self.blocks_pdu_to_tagged_stream_0 = (
            pdu_to_tagged_stream(byte_t, 'packet_len'))

        ##################################################
        # Connections
        ##################################################
        self.msg_connect(
            (self.blocks_tagged_stream_to_pdu_0_0_0_0_0, 'pdus'),
            (self, 'out'))
        self.msg_connect(
            (self, 'in'), (self.blocks_pdu_to_tagged_stream_0, 'pdus'))
        self.connect(
            (self.blocks_pdu_to_tagged_stream_0, 0),
            (self.digital_additive_scrambler_bb_0_0, 0))
        self.connect(
            (self.blocks_tagged_stream_multiply_length_0_0_0_0, 0),
            (self.blocks_tagged_stream_to_pdu_0_0_0_0_0, 0))
        self.connect(
            (self.blocks_unpacked_to_packed_xx_0_0_0_0, 0),
            (self.blocks_tagged_stream_multiply_length_0_0_0_0, 0))
        self.connect(
            (self.digital_additive_scrambler_bb_0_0, 0),
            (self.blocks_unpacked_to_packed_xx_0_0_0_0, 0))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr_unittest

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites.telemetry.by02 import frameB


class qa_by02(gr_unittest.TestCase):
    def test_frameB(self):
        """Tries to parse a frameB"""
        frame = """08 10 2c 2d 00 55 55 55 55 55 55 00 26 e7 90 ff
ff ff ff 00 00 00 00 51 ee 65 00 01 00 49 4e 30
4f 52 59 0a 49 11 29 00 c8 d7 43 00 00 55 ff 00
00 1b 56 aa aa aa aa aa aa aa aa aa aa aa aa aa
aa aa aa aa aa aa aa aa aa aa aa aa aa aa aa aa
aa"""
        frame = bytes().fromhex(frame.replace(' ', '').replace('\n', ''))
        data = frameB.parse(frame)
        self.assertEqual(data.header.spacecraft_id, 129,
                         'TM header spacecraft ID is not correct')
        self.assertEqual(data.header.master_channel_frame_count, 44,
                         'TM header master channel frame count is not correct')
        self.assertEqual(
            data.header.virtual_channel_frame_count, 45,
            'TM header virtual channel frame count is not correct')
        self.assertEqual(data.header.virtual_channel_id, 0,
                         'TM header virtual channel ID is not correct')
        self.assertEqual(data.hk_AVR.callsign, b'IN0ORY',
                         'Decoded callsign is not correct')
        self.assertEqual(frame, frameB.build(data),
                         'Rebuilt frame does not equal original frame')


if __name__ == '__main__':
    gr_unittest.run(qa_by02)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2021 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import convolutional_encoder, viterbi_decoder


class qa_viterbi(gr_unittest.TestCase):
    def test_viterbi(self):
        tb = gr.top_block()
        dbg = blocks.message_debug()
        k = 5
        p = [25, 23]
        enc = convolutional_encoder(k, p)
        dec = viterbi_decoder(k, p)
        data = np.random.randint(2, size=1000, dtype='uint8')
        pdu = pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(data), data))

        tb.msg_connect((enc, 'out'), (dec, 'in'))
        tb.msg_connect((dec, 'out'), (dbg, 'store'))
        enc.to_basic_block()._post(pmt.intern('in'), pdu)
        enc.to_basic_block()._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))

        tb.start()
        tb.wait()

        out = pmt.u8vector_elements(pmt.cdr(dbg.get_message(0)))
        np.testing.assert_equal(
            out, np.array(data),
            'Encoded and decoded message does not match original')


if __name__ == '__main__':
    gr_unittest.run(qa_viterbi)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites.hier import rms_agc


class qa_rms_agc(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_zero_input(self):
        """Tests that if the input is zero, the output is also zero.

        In some architectures we have found that the calculations produce nan
        if the number that we put in the denominator to avoid 0/0 is too small.
        """
        n = 256
        source = blocks.vector_source_c([0] * n, False, 1, [])
        sink = blocks.vector_sink_c(1, n)
        agc = rms_agc(alpha=1e-2, reference=1.0)
        self.tb.connect(source, agc, sink)

        self.tb.start()
        self.tb.wait()

        expected = np.zeros(n, 'complex64')
        np.testing.assert_equal(
            sink.data(), expected, 'AGC output is not zero')


if __name__ == '__main__':
    gr_unittest.run(qa_rms_agc)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import struct

from gnuradio import gr, digital
import pmt

from ...crcs import crc16_ccitt_zero
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block


class spino_crop(gr.basic_block):
    """
    Helper block to crop SPINO frames to the correct length.
    """
    def __init__(self):
        gr.basic_block.__init__(
            self,
            name='spino_crop',
            in_sig=[],
            out_sig=[])
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print('[ERROR] Received invalid message type. Expected u8vector')
            return
        msg = pmt.u8vector_elements(msg)
        length = struct.unpack('<H', bytes(msg[16:18]))[0]
        # account for AX.25 headers (14 bytes) and CRC (2 bytes)
        length += 16
        self.message_port_pub(
            pmt.intern('out'),
            pmt.cons(pmt.car(msg_pmt), pmt.init_u8vector(length, msg)))


_syncword = '00101110111111001001100000100111'


class spino_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe SPINO packets

    The input is a float stream of soft symbols. The output are PDUs
    with the decoded frames. This uses frames of a fixed size, a length
    field to determine the length of the payload (the remaining data in
    the frame is padding, and a CRC-16).

    https://code.electrolab.fr/spino/cubesat_cs/-/blob/master/
    SPECIFICATIONS/2023_04_01%20-%20Pr%C3%A9sentation%20Spino%20-%20CJ.pdf

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold=None, options=None):
        gr.hier_block2.__init__(
            self,
            'spino_deframer',
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)

        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        self.deframer = sync_to_pdu_packed(
            packlen=240, sync=_syncword, threshold=syncword_threshold)
        self.crop = spino_crop()
        self.crc_check = crc16_ccitt_zero(swap_endianness=True)

        self.connect(self, self.slicer, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.crop, 'in'))
        self.msg_connect((self.crop, 'out'), (self.crc_check, 'in'))
        self.msg_connect((self.crc_check, 'ok'), (self, 'out'))

    _default_sync_threshold = 0

    @classmethod
    def add_options(cls, parser):
        """
        Adds SPINO deframer specific options to the argparse parser
        """
        parser.add_argument(
            '--syncword_threshold', type=int,
            default=cls._default_sync_threshold,
            help='Syncword bit errors [default=%(default)r]')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
import pmt

from ... import decode_rs, pdu_head_tail
from ...crcs import crc16_ccitt_false
from .ccsds_rs_deframer import _syncword
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block

# 0xB22B in MSB first
_syncword = '1011010000101011'


class sanosat_packet_crop(gr.basic_block):
    """Crop SanoSat-1 packet"""
    def __init__(self):
        gr.basic_block.__init__(
            self,
            name='sanosat_packet_crop',
            in_sig=[],
            out_sig=[])
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print('[ERROR] Received invalid message type. Expected u8vector')
            return
        packet = bytes(pmt.u8vector_elements(msg))
        # 5 bytes consists of 1 byte for lenght field
        # plus 4 bytes for delimiter FFFF0000
        packet_length = packet[0] + 5
        packet = bytearray(packet[:packet_length])
        # Delete CRC1 field, which is not used in the CRC2 calculation
        del packet[1:3]
        self.message_port_pub(
            pmt.intern('out'),
            pmt.cons(pmt.car(msg_pmt),
                     pmt.init_u8vector(len(packet), list(packet))))


class sanosat_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe SanoSat-1 custom frames

    See https://amsat-np.org/transmission-protocol-for-sanosat-1/

    The input is a float stream of soft symbols. The output are PDUs
    with frames.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold=None, options=None):
        gr.hier_block2.__init__(
            self,
            'sanosat_deframer',
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)

        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        self.deframer = sync_to_pdu_packed(
            packlen=135, sync=_syncword,
            threshold=syncword_threshold)
        self.crop = sanosat_packet_crop()
        self.crc = crc16_ccitt_false(swap_endianness=True)
        # Remove length field and FFFF0000 delimiter
        self.crop2 = pdu_head_tail(3, 5)

        self.connect(self, self.slicer, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.crop, 'in'))
        self.msg_connect((self.crop, 'out'), (self.crc, 'in'))
        self.msg_connect((self.crc, 'ok'), (self.crop2, 'in'))
        self.msg_connect((self.crop2, 'out'), (self, 'out'))

    _default_sync_threshold = 0

    @classmethod
    def add_options(cls, parser):
        """
        Adds SanoSat-1 deframer specific options to the argparse parser
        """
        parser.add_argument(
            '--syncword_threshold', type=int,
            default=cls._default_sync_threshold,
            help='Syncword bit errors [default=%(default)r]')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import pdu_scrambler


class qa_pdu_scrambler(gr_unittest.TestCase):
    def test_pdu_scrambler(self):
        self.tb = gr.top_block()
        self.dbg = blocks.message_debug()
        size = 1024
        sequence = np.random.randint(0, 256, size)
        npdus = 16
        msgs = [np.random.randint(0, 256, size=np.random.randint(0, size),
                                  dtype='uint8')
                for _ in range(npdus)]
        pdus = [pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(m), m))
                for m in msgs]
        self.scrambler = pdu_scrambler(sequence)
        self.tb.msg_connect((self.scrambler, 'out'), (self.dbg, 'store'))
        for pdu in pdus:
            self.scrambler.to_basic_block()._post(pmt.intern('in'), pdu)
        self.scrambler.to_basic_block()._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))

        self.tb.run()

        for j, msg in enumerate(msgs):
            out = pmt.u8vector_elements(pmt.cdr(self.dbg.get_message(j)))
            np.testing.assert_equal(out, msg ^ sequence[:msg.size])

        self.tb = None


if __name__ == '__main__':
    gr_unittest.run(qa_pdu_scrambler)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
import pmt

from ... import decode_rs, pdu_head_tail
from ...crcs import crc16_ccitt_false
from .ccsds_rs_deframer import _syncword
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block


# 0xBF35 in MSB first
_syncword = '1011111100110101'


class hades_packet_crop(gr.basic_block):
    """Crop HADES-D packet"""
    def __init__(self):
        gr.basic_block.__init__(
            self,
            name='hades_packet_crop',
            in_sig=[],
            out_sig=[])
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print('[ERROR] Received invalid message type. Expected u8vector')
            return
        packet = bytes(pmt.u8vector_elements(msg))
        packet_type = packet[0] >> 4
        lengths = {1: 26, 2: 13, 3: 26, 4: 54, 5: 33, 6: 135, 7: 67, 8: 28,
                   9: 123, 12: 64}
        try:
            packet_length = lengths[packet_type]
        except KeyError:
            # invalid packet_type
            return
        packet = packet[:packet_length]
        self.message_port_pub(
            pmt.intern('out'),
            pmt.cons(pmt.car(msg_pmt),
                     pmt.init_u8vector(len(packet), list(packet))))


class hades_descrambler(gr.basic_block):
    """Descramble HADES-D packet"""
    def __init__(self):
        gr.basic_block.__init__(
            self,
            name='hades_packet_crop',
            in_sig=[],
            out_sig=[])
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print('[ERROR] Received invalid message type. Expected u8vector')
            return
        packet = bytearray(bytes(pmt.u8vector_elements(msg)))
        # The algorithm deviates from a typical scrambler (and in fact the
        # implementation has some bugs), so it is impossible to get it right
        # just by looking at the documentation.
        #
        # See
        # https://github.com/AMSAT-EA/URESAT-1-decoder/blob/main/bit-version/linux/genesis_scrambler.c
        # for the AMSAT-EA implementation.

        # In the AMSAT-EA code the state is initialized to 0x2C350000, but
        # since the shift register is supposed to have 17 bits, most of the
        # non-zero MSBs do not matter.
        state = 1 << 16
        # The first byte is not scrambled (the CRC is not scrambled either, but
        # it has been removed at this point)
        for j in range(1, len(packet)):
            out = 0
            # There is a bug in how the scrambler is written.  The LSB is each
            # byte is not scrambler, and the scrambler state is not advanced.
            for k in range(7):
                b = (packet[j] >> (7 - k)) & 1
                b0 = b
                b ^= ((state >> 16) ^ (state >> 11)) & 1
                out = (out << 1) | b
                state = ((state << 1) | b0) & 0x1ffff
            # Take LSB directly without scrambling
            out = (out << 1) | (packet[j] & 1)
            packet[j] = out
        self.message_port_pub(
            pmt.intern('out'),
            pmt.cons(pmt.car(msg_pmt),
                     pmt.init_u8vector(len(packet), list(packet))))


class hades_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe HADES-D custom frames

    https://www.amsat-ea.org/app/download/13595424/AMSAT+EA+-+Descripci%C3%B3n+de+transmisiones+de+HADES-D.pdf
    https://www.amsat-ea.org/app/download/13595777/AMSAT+EA+-+HADES-D+Transmissions+description.pdf

    The input is a float stream of soft symbols. The output are PDUs
    with frames.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold=None, options=None):
        gr.hier_block2.__init__(
            self,
            'hades_deframer',
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)

        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        self.deframer = sync_to_pdu_packed(
            packlen=135, sync=_syncword,
            threshold=syncword_threshold)
        self.crop = hades_packet_crop()
        self.crc = crc16_ccitt_false()
        self.descrambler = hades_descrambler()

        self.connect(self, self.slicer, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.crop, 'in'))
        self.msg_connect((self.crop, 'out'), (self.crc, 'in'))
        self.msg_connect((self.crc, 'ok'), (self.descrambler, 'in'))
        self.msg_connect((self.descrambler, 'out'), (self, 'out'))

    _default_sync_threshold = 0

    @classmethod
    def add_options(cls, parser):
        """
        Adds HADES-D deframer specific options to the argparse parser
        """
        parser.add_argument(
            '--syncword_threshold', type=int,
            default=cls._default_sync_threshold,
            help='Syncword bit errors [default=%(default)r]')



# Copyright 2017, 2018, 2019, 2020 Daniel Estevez <daniel@destevez.net>
# Copyright 2022 The Regents of the University of Colorado
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from .ax25 import Header
from ..ccsds import space_packet as ccsds_space_packet
from ..adapters import PolynomialAdapter
from construct import Adapter, BitsInteger, BitStruct, Bytes, Container, \
                Default, Enum, Flag, Float32l, Hex, If, Int8ub, Int8sl, \
                Int8ul, Int16sl, Int16ul, Int16sb, Int16ub, Int32ub, \
                Int32ul, Int32sl, Int32sb, Padding, Struct, Switch

SSID = BitStruct(
    'ch' / Flag,  # C / H bit
    Default(BitsInteger(2), 3),  # reserved bits
    'ssid' / BitsInteger(4),
    'extension' / Flag  # last address bit
    )


class CallsignAdapter(Adapter):
    def _encode(self, obj, context, path=None):
        return bytes([x << 1 for x in bytes(
            (obj.upper() + ' '*6)[:6], encoding='ascii')])

    def _decode(self, obj, context, path=None):
        return str(bytes([x >> 1 for x in obj]), encoding='ascii').strip()


Callsign = CallsignAdapter(Bytes(6))
CallsignUnshift = Bytes(6)
Control = Hex(Int8ub)
PID = Hex(Int8ub)

Address = Struct(
        'callsign' / Callsign,
        'ssid' / SSID
        )

UnshiftAddress = Struct(
        'callsign' / CallsignUnshift,
        'ssid' / SSID
        )

Header = Struct(
    'dest' / Address,
    'src' / UnshiftAddress,
    'control' / Control,
    'pid' / PID
    )

SecondaryHeader = Struct(
    'sh_coarse' / Int32ul,
    'sh_fine' / Int16ub,
)

cmd_opcodes = Enum(
        Int16ub,
        NOOP=0, ARM=1, CMD_RST_STATS=2, CMD_XSUM=3,
        CMD_ECHO_STATE=4, CMD_VERSION=5, LOG_ROUTE=48,
        LOG_STATE=49, LOG_RST_STATS=50, LOG_RESET_READ=52,
        LOG_ISSUE=53, LOG_DUMP_INFO=54, LOG_SET_PUBLISH=55, MEM_DUMP=80,
        MEM_LOAD=81, MEM_ERASE=82, MEM_XSUM=83, MEM_ABORT=84,
        MEM_RESET=85, MEM_LOAD_DWORD=86, MEM_LOAD_WORD=87,
        MEM_LOAD_BYTE=88, PKT_ISSUE=96, PKT_SET_RATE=97,
        PKT_SET_STREAM=98, PKT_SET_PRIORITY=99, PKT_QUERY_APID=100,
        PKT_SET_DELAY=101, PKT_COMMIT_TABLE=102, TBL_DUMP=112,
        TBL_LOAD=113, TBL_LOAD_START=114, TBL_COMMIT=115, TBL_ABORT=116,
        TBL_VERIFY=117, TBL_SAVE=118, DES_RST_STATS=33,
        DES_ADD_TASK=34, DES_SUB_TASK=35, DES_ADD_BACK=36,
        DES_SUB_BACK=37, DES_SET_TIME=38, DES_DUMP_SCHED=39,
        DES_SET_TASK_NUM=40, SEQ_LOAD=64, SEQ_INIT=65, SEQ_STATE=66,
        SEQ_VERIFY=67, SEQ_STOP=68, SEQ_FIND=69, SEQ_INFO=70,
        SEQ_DUMP=71, SEQ_RST_STATS=72, SEQ_LD_START=73, SEQ_LD_COMMIT=74,
        SEQ_LD_ABORT=75, SEQ_LIBRARY=76, FP_RST_STATS=16,
        FP_RST_RESULTS=17, FP_VALIDATE=18, FP_SET_STATE=19,
        FP_SET_WP_STATE=20, FP_DUMP_RESULTS=22, FP_SET_WP=23,
        FP_SET_WP_THRESH=24, FP_SET_WP_RESP=25, FP_SET_TEST=26,
        CMD_OPCODE_UHF_PASS=128, CMD_OPCODE_UHF_INIT=129,
        CMD_OPCODE_UHF_RESP_STATE=130, ADCS_PASS=144, ADCS_RESET=145,
        ADCS_READ=146, ADCS_COARSE_POINT=147, ADCS_FINE_POINT=148,
        ADCS_RAM_POINT=149, ADCS_FINE_UPDATE=150, ADCS_RAM_UPDATE=151,
        ADCS_ECLIPSE_CHECK=152, ADCS_DUMP_IMAGE=153,
        ADCS_DUMP_STATE=154, ADCS_INIT_EPHEMERIS=155, ADCS_SET_TIME=156,
        CMD_OPCODE_CFI_PWR_ON=224, CMD_OPCODE_CFI_PWR_OFF=225,
        CMD_OPCODE_CFI_SELECT=226, CMD_OPCODE_CFI_SELECT_TRIO=227,
        STORE_WRITE_STATE=240, STORE_READ=241, STORE_HALT=242,
        STORE_PLAYBACK=243, STORE_SELECT=245, STORE_SET_PARTITION=246,
        STORE_RESET_PARTITION=247, STORE_FLUSH=248,
        STORE_MOVE_READ=249, PAYLOAD_MSG_STATE=208, PAYLOAD_HK_RESET=209,
        PAYLOAD_FLAGS_RESET=210, EPS_SET_BAT_TIME=203,
        EPS_GET_BAT_TIME=204, EPS_BAT_PASS=205, EPS_BAT_TLM_STATE=206,
        EPS_BAT_HEATER=207, EPS_PWR_ON=192, EPS_PWR_OFF=193,
        EPS_PWR_CYCLE=194, EPS_DEPLOY=195, EPS_ANT_BURN=196,
        EPS_ANT_HALT_BURN=197, EPS_ANT_SET_BACKUP=198, EPS_ANT_RESET=199,
        EPS_DEPLOY_ABORT=200, EPS_ANT_TLM=201, EPS_RELAY=202,
        MRAM_CLEAR=160, MRAM_ERR_INJ=161, MRAM_PWR=162,
        MRAM_PWR_TOGGLE=163, FLASH_SEL_PAYLOAD=164, CLT_RESET=239,
        CLT_THRESHOLD=238, WATCHDOG=237, PHOENIX=236, SAFE=235,
        NOMINAL=234, NAND_BB_INIT=169, NAND_REMAP=170, NAND_BAD_BLOCK=171,
        NAND_FORMAT=172, NAND_FORMAT_HALT=173, SBAND_SYNC_ON=176,
        SBAND_SYNC_OFF=177, SBAND_CLOCK=178)

seq_opcodes = Enum(
        Int16ub,
        RAM_SMALL0=0, RAM_SMALL1=1, RAM_SMALL2=2,
        RAM_SMALL3=3, RAM_SMALL4=4, RAM_SMALL5=5,
        RAM_SMALL6=6, RAM_SMALL7=7, RAM_LARGE0=8, RAM_LARGE1=9,
        RAM_LARGE2=10, RAM_LARGE3=11, NVM_SMALL0=12, NVM_SMALL1=13,
        NVM_SMALL2=14, NVM_SMALL3=15, NVM_SMALL4=16, NVM_SMALL5=17,
        NVM_SMALL6=18, NVM_SMALL7=19, NVM_SMALL8=20, NVM_SMALL9=21,
        NVM_SMALL10=22, NVM_SMALL11=23, NVM_SMALL12=24, NVM_SMALL13=25,
        NVM_SMALL14=26, NVM_SMALL15=27, NVM_SMALL16=28,
        NVM_SMALL17=29, NVM_SMALL18=30, NVM_SMALL19=31, NVM_SMALL20=32,
        NVM_SMALL21=33, NVM_SMALL22=34, NVM_SMALL23=35,
        NVM_SMALL24=36, NVM_SMALL25=37, NVM_SMALL26=38, NVM_SMALL27=39,
        NVM_SMALL28=40, NVM_SMALL29=41, NVM_SMALL30=42,
        NVM_SMALL31=43, NVM_SMALL32=44, NVM_SMALL33=45, NVM_SMALL34=46,
        NVM_SMALL35=47, NVM_SMALL36=48, NVM_SMALL37=49,
        NVM_SMALL38=50, NVM_SMALL39=51, NVM_SMALL40=52, NVM_SMALL41=53,
        NVM_SMALL42=54, NVM_SMALL43=55, NVM_SMALL44=56,
        NVM_SMALL45=57, NVM_SMALL46=58, NVM_SMALL47=59, NVM_SMALL48=60,
        NVM_SMALL49=61, NVM_SMALL50=62, NVM_SMALL51=63,
        NVM_SMALL52=64, NVM_SMALL53=65, NVM_SMALL54=66, NVM_SMALL55=67,
        NVM_SMALL56=68, NVM_SMALL57=69, NVM_SMALL58=70,
        NVM_SMALL59=71, NVM_SMALL60=72, NVM_SMALL61=73, NVM_SMALL62=74,
        NVM_SMALL63=75, NVM_SMALL64=76, NVM_SMALL65=77,
        NVM_SMALL66=78, NVM_SMALL67=79, NVM_SMALL68=80, NVM_SMALL69=81,
        NVM_SMALL70=82, NVM_SMALL71=83, NVM_SMALL72=84,
        NVM_SMALL73=85, NVM_SMALL74=86, NVM_SMALL75=87, NVM_SMALL76=88,
        NVM_SMALL77=89, NVM_SMALL78=90, NVM_SMALL79=91,
        NVM_SMALL80=92, NVM_SMALL81=93, NVM_SMALL82=94, NVM_SMALL83=95,
        NVM_SMALL84=96, NVM_SMALL85=97, NVM_SMALL86=98,
        NVM_SMALL87=99, NVM_SMALL88=100, NVM_SMALL89=101, NVM_SMALL90=102,
        NVM_SMALL91=103, NVM_SMALL92=104, NVM_SMALL93=105,
        NVM_SMALL94=106, NVM_SMALL95=107, NVM_SMALL96=108,
        NVM_SMALL97=109, NVM_SMALL98=110, NVM_SMALL99=111,
        NVM_SMALL100=112, NVM_SMALL101=113, NVM_SMALL102=114,
        NVM_SMALL103=115, NVM_SMALL104=116, NVM_SMALL105=117,
        NVM_SMALL106=118, NVM_SMALL107=119, NVM_SMALL108=120,
        NVM_SMALL109=121, NVM_SMALL110=122, NVM_SMALL111=123,
        NVM_SMALL112=124, NVM_SMALL113=125, NVM_SMALL114=126,
        NVM_SMALL115=127, NVM_SMALL116=128, NVM_SMALL117=129,
        NVM_SMALL118=130, NVM_SMALL119=131, NVM_SMALL120=132,
        NVM_SMALL121=133, NVM_SMALL122=134, NVM_SMALL123=135,
        NVM_SMALL124=136, NVM_SMALL125=137, NVM_SMALL126=138,
        NVM_SMALL127=139, HOLDING0=140)

alive_arm_state = BitStruct(
    'reusable_spare_1' / BitsInteger(1),
    'reusable_spare_2' / BitsInteger(1),
    'reusable_spare_3' / BitsInteger(1),
    'reusable_spare_4' / BitsInteger(1),
    'reusable_spare_5' / BitsInteger(1),
    'sw_cmd_arm_state_uhf' / Enum(BitsInteger(1), OFF=0, ARMED=1),
    'sw_cmd_arm_state_seq' / Enum(BitsInteger(1), OFF=0, ARMED=1),
    'sw_cmd_arm_state_ext' / Enum(BitsInteger(1), OFF=0, ARMED=1),
    'reusable_spare_6' / BitsInteger(1),
    'sw_eps_pwr_state_deploy_pwr' / Enum(BitsInteger(1), OFF=0, ON=1),
    'sw_eps_pwr_state_deploy' / Enum(BitsInteger(1), OFF=0, ON=1),
    'sw_eps_pwr_state_iic' / Enum(BitsInteger(1), OFF=0, ON=1),
    'sw_eps_pwr_state_inst' / Enum(BitsInteger(1), OFF=0, ON=1),
    'sw_eps_pwr_state_sband' / Enum(BitsInteger(1), OFF=0, ON=1),
    'sw_eps_pwr_state_uhf' / Enum(BitsInteger(1), OFF=0, ON=1),
    'sw_eps_pwr_state_adcs' / Enum(BitsInteger(1), OFF=0, ON=1),
    'reusable_spare_7' / BitsInteger(1),
    'reusable_spare_8' / BitsInteger(1),
    'reusable_spare_9' / BitsInteger(1),
    'reusable_spare_10' / BitsInteger(1),
    'reusable_spare_11' / BitsInteger(1),
    'reusable_spare_12' / BitsInteger(1),
    'sw_bat_alive_state_battery1' / Enum(BitsInteger(1), DEAD=0, ALIVE=1),
    'sw_bat_alive_state_battery0' / Enum(BitsInteger(1), DEAD=0, ALIVE=1)
)

payload_state = BitStruct(
    'spare_2' / BitsInteger(2),
    'sw_payload_pwr_cycle_req' / Enum(BitsInteger(1), INACTIVE=0, ACTIVE=1),
    'sw_payload_pwr_off_req' / Enum(BitsInteger(1), INACTIVE=0, ACTIVE=1),
    'sw_payload_stat_msg_state' / Enum(BitsInteger(1), DIS=0, ENA=1),
    'sw_payload_time_msg_state' / Enum(BitsInteger(1), DIS=0, ENA=1),
    'sw_payload_alive_state' / Enum(BitsInteger(2), OFF=0, DEAD=1, ALIVE=2)
)

shutter_state = BitStruct(
    'sw_shutter_state_b4_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_b3_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_b2_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_b1_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_a4_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_a3_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_a2_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1),
    'sw_shutter_state_a1_ctim' / Enum(BitsInteger(1), OPEN=0, CLOSE=1)
)

adcs_info = BitStruct(
    'sw_adcs_att_valid' / Enum(BitsInteger(1), YES=1, NO=0),
    'sw_adcs_ref_valid' / Enum(BitsInteger(1), YES=1, NO=0),
    'sw_adcs_time_valid' / Enum(BitsInteger(1), YES=1, NO=0),
    'sw_adcs_mode' / Enum(BitsInteger(1), FINE_REF_POINT=1, SUN_POINT=0),
    'sw_adcs_recommend_sun_point' / Enum(BitsInteger(1), YES=1, NO=0),
    'sw_adcs_sun_point_state' / Enum(BitsInteger(3), SEARCHING=3, WAITING=4,
                                     CONVERGING=5, NOT_ACTIVE=7, ON_SUN=6,
                                     SEARCH_INIT=2)
)

ctim_beacon = Struct(
    'sw_major_version' / Int8ul,
    'sw_minor_version' / Int8ul,
    'sw_patch_version' / Int8ul,
    'sw_image_id' / Int8ul,
    'sw_cmd_recv_count' / Int16ub,
    'sw_cmd_fmt_count' / Int16ub,
    'sw_cmd_rjct_count' / Int16ub,
    'sw_cmd_succ_count' / Int16ub,
    'sw_cmd_succ_op' / cmd_opcodes,
    'sw_cmd_rjct_op' / cmd_opcodes,
    'sw_cmd_fail_code' / Enum(Int8ul, SUCCESS=0, MODE=1, ARM=2,
                              SOURCE=3, OPCODE=4, METHOD=5,
                              LENGTH=6, RANGE=7, CHECKSUM=8,
                              PKT_TYPE=9),
    'sw_cmd_xsum_state' / Enum(Int8ul, DIS=0, ENA=1),
    'alive_arm_state' / alive_arm_state,
    'sw_mode_clt_count' / Int8ul,
    'sw_mode_system_mode' / Enum(Int8ul, PHOENIX=0, SAFE=1, NOMINAL=2),
    'sw_sband_sync_state' / Enum(Int8ul, DIS=0, ENA=1),
    'sw_time_since_rx' / Int16ub,
    'sw_sband_timeout' / Int16ub,
    'payload_state' / payload_state,
    'sw_uhf_alive' / Int8ul,
    'sw_uhf_temp' / Int8sl,
    'sw_adcs_alive' / Enum(Int8ul, OFF=0, DEAD=1, ALIVE=2),
    'sw_inst_cmd_succ_count_ctim' / Int16ub,
    'sw_inst_cmd_rjct_count_ctim' / Int8ul,
    'sw_esr_obs_id_ctim' / Int8ul,
    'sw_thrm_a1_a_ctim' / Int16ub,
    'sw_thrm_a1_b_ctim' / Int16ub,
    'sw_fss_q1_ctim' / Int16ub,
    'sw_fss_q2_ctim' / Int16ub,
    'sw_fss_q3_ctim' / Int16ub,
    'sw_fss_q4_ctim' / Int16ub,
    'sw_volt_p12v_ctim' / Int16ub,
    'sw_thrm_pwm_ctim' / Int16ub,
    'sw_inst_fp_resp_count_ctim' / Int16ub,
    'shutter_state' / shutter_state,
    'sw_inst_cmd_fail_code_ctim' / Enum(Int8ul, SUCCESS=0, MODE=1,
                                        ARM=2, SOURCE=3, OPCODE=4,
                                        METHOD=5, LENGTH=6, RANGE=7,
                                        CHECKSUM=8, PKT_TYPE=9),
    'sw_esr_filtered_a12_ctim' / Float32l,
    'sw_esr_filtered_b12_ctim' / Float32l,
    'sw_seq_state_auto' / Enum(Int8ul, IDLE=0, ACTIVE=1,
                               SUSPEND=2, PAUSE=3, STALE=4),
    'sw_seq_state_op1' / Enum(Int8ul, IDLE=0, ACTIVE=1,
                              SUSPEND=2, PAUSE=3, STALE=4),
    'sw_seq_state_op2' / Enum(Int8ul, IDLE=0, ACTIVE=1,
                              SUSPEND=2, PAUSE=3, STALE=4),
    'sw_seq_state_op3' / Enum(Int8ul, IDLE=0, ACTIVE=1,
                              SUSPEND=2, PAUSE=3, STALE=4),
    'sw_seq_stop_code_auto' / Enum(Int8ul, NOMINAL=0, CMD=1, INIT=2,
                                   REJECT=3, STALE=4, BAD_INT=5,
                                   INT_FAIL=6),
    'sw_seq_stop_code_op1' / Enum(Int8ul, NOMINAL=0, CMD=1, INIT=2,
                                  REJECT=3, STALE=4, BAD_INT=5,
                                  INT_FAIL=6),
    'sw_seq_stop_code_op2' / Enum(Int8ul, NOMINAL=0, CMD=1, INIT=2,
                                  REJECT=3, STALE=4, BAD_INT=5,
                                  INT_FAIL=6),
    'sw_seq_stop_code_op3' / Enum(Int8ul, NOMINAL=0, CMD=1, INIT=2,
                                  REJECT=3, STALE=4, BAD_INT=5,
                                  INT_FAIL=6),
    'sw_seq_exec_buf_auto' / seq_opcodes,
    'sw_seq_exec_buf_op1' / seq_opcodes,
    'sw_seq_exec_buf_op2' / seq_opcodes,
    'sw_seq_exec_buf_op3' / seq_opcodes,
    'sw_store_partition_write_misc' / Int32ub,
    'sw_store_partition_read_misc' / Int32ub,
    'sw_store_partition_write_adcs' / Int32ub,
    'sw_store_partition_read_adcs' / Int32ub,
    'sw_store_partition_write_beacon' / Int32ub,
    'sw_store_partition_read_beacon' / Int32ub,
    'sw_store_partition_write_log' / Int32ub,
    'sw_store_partition_read_log' / Int32ub,
    'sw_store_partition_write_payload' / Int32ub,
    'sw_store_partition_read_payload' / Int32ub,
    'sw_fp_resp_count' / Int16ub,
    'sw_ana_bus_v' / PolynomialAdapter([0.0, 0.008862], Int16ub),
    'sw_ana_3p3_v' / PolynomialAdapter([0.0, 0.001611], Int16ub),
    'sw_ana_3p3_i' / PolynomialAdapter([0.0, 8.1e-05], Int16ub),
    'sw_ana_1p8_i' / PolynomialAdapter([0.0, 8.1e-05], Int16ub),
    'sw_ana_1p0_i' / PolynomialAdapter([0.0, 8.1e-05], Int16ub),
    'sw_ana_cdh_temp' / PolynomialAdapter([98.0, -0.05936, 1.641e-05,
                                          -0.000000002361], Int16ub),
    'sw_ana_sa1_v' / PolynomialAdapter([0.0, 0.009659], Int16ub),
    'sw_ana_sa1_i' / PolynomialAdapter([-0.04241, 0.002525], Int16ub),
    'sw_ana_sa2_v' / PolynomialAdapter([0.0, 0.009659], Int16ub),
    'sw_ana_sa2_i' / PolynomialAdapter([-0.04262, 0.002525], Int16ub),
    'sw_ana_bat1_v' / PolynomialAdapter([0.0, 0.008862], Int16ub),
    'sw_ana_bat2_v' / PolynomialAdapter([0.0, 0.008862], Int16ub),
    'sw_ana_eps_temp' / PolynomialAdapter([98.0, -0.05936, 1.641e-05,
                                          -0.000000002361], Int16ub),
    'sw_ana_eps_3p3_ref' / PolynomialAdapter([0.0, 0.001611], Int16ub),
    'sw_ana_eps_bus_i' / PolynomialAdapter([0.0, 0.001221], Int16ub),
    'sw_ana_xact_i' / PolynomialAdapter([-0.02489, 0.001992], Int16ub),
    'sw_ana_uhf_i' / PolynomialAdapter([-0.02489, 0.001992], Int16ub),
    'sw_ana_sband_i' / PolynomialAdapter([-0.02489, 0.001992], Int16ub),
    'sw_ana_inst_i' / PolynomialAdapter([0.0, 0.000807], Int16ub),
    'sw_ana_hk_3p3_ref' / PolynomialAdapter([0.0, 0.001611], Int16ub),
    'sw_ana_ifb_i' / PolynomialAdapter([0.0, 0.000269], Int16ub),
    'sw_ana_ifb_temp' / PolynomialAdapter([98.0, -0.05936, 1.641e-05,
                                          -0.000000002361], Int16ub),
    'sw_adcs_eclipse' / Int8ul,
    'adcs_info' / adcs_info,
    'sw_adcs_analogs_voltage_2p5' / PolynomialAdapter([0.0, 0.015], Int8ul),
    'sw_adcs_analogs_voltage_1p8' / PolynomialAdapter([0.0, 0.015], Int8ul),
    'sw_adcs_analogs_voltage_1p0' / PolynomialAdapter([0.0, 0.015], Int8ul),
    'sw_adcs_analogs_det_temp' / PolynomialAdapter([0.0, 0.8], Int8sl),
    'sw_adcs_analogs_motor1_temp' / PolynomialAdapter([0.0, 0.005], Int16sb),
    'sw_adcs_analogs_motor2_temp' / PolynomialAdapter([0.0, 0.005], Int16sb),
    'sw_adcs_analogs_motor3_temp' / PolynomialAdapter([0.0, 0.005], Int16sb),
    'spare_16' / Int16sl,
    'sw_adcs_analogs_digital_bus_v' / PolynomialAdapter([0.0, 0.00125],
                                                        Int16sb),
    'sw_adcs_cmd_acpt' / Int8ul,
    'sw_adcs_cmd_fail' / Int8ul,
    'sw_adcs_time' / Int32sl,
    'sw_adcs_sun_vec1' / PolynomialAdapter([0.0, 0.0001], Int16sb),
    'sw_adcs_sun_vec2' / PolynomialAdapter([0.0, 0.0001], Int16sb),
    'sw_adcs_sun_vec3' / PolynomialAdapter([0.0, 0.0001], Int16sb),
    'sw_adcs_wheel_sp1' / PolynomialAdapter([0.0, 0.4], Int16sb),
    'sw_adcs_wheel_sp2' / PolynomialAdapter([0.0, 0.4], Int16sb),
    'sw_adcs_wheel_sp3' / PolynomialAdapter([0.0, 0.4], Int16sb),
    'sw_adcs_body_rt1' / PolynomialAdapter([0.0, 5E-9], Int32sb),
    'sw_adcs_body_rt2' / PolynomialAdapter([0.0, 5E-9], Int32sb),
    'sw_adcs_body_rt3' / PolynomialAdapter([0.0, 5E-9], Int32sb),
    'sw_adcs_body_quat1' / PolynomialAdapter([0.0, 5E-10], Int32sb),
    'sw_adcs_body_quat2' / PolynomialAdapter([0.0, 5E-10], Int32sb),
    'sw_adcs_body_quat3' / PolynomialAdapter([0.0, 5E-10], Int32sb),
    'sw_adcs_body_quat4' / PolynomialAdapter([0.0, 5E-10], Int32sb),
    'sw_spare_0' / Int32ul,
    'sw_spare_1' / Int32ul,
    'sw_spare_2' / Int32ul,
    'sw_spare_3' / Int32ul
)


ctim_70cm = Struct(
    'ax25_header' / Header,
    'primary_header' / ccsds_space_packet.PrimaryHeader,
    'secondary_header' / If(
        lambda c: c.primary_header.secondary_header_flag,
        SecondaryHeader
    ),
    'packet' / Switch(
        lambda c: c.primary_header.AP_ID,
        {
            0x01: ctim_beacon
        }
    )
)



# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: LilacSat-1 decoder
# Author: Daniel Estevez
# Description: LilacSat-1 decoder
# Generated: Sun Aug 20 10:39:10 2017
##################################################

import os
import sys
sys.path.append(os.environ.get('GRC_HIER_PATH', os.path.expanduser('~/.grc_gnuradio')))

from ccsds_viterbi import ccsds_viterbi  # grc-generated hier_block
from gnuradio import blocks
from gnuradio import digital
from gnuradio import filter
from gnuradio import gr
from gnuradio.filter import firdes
from gnuradio.fft import window
from rms_agc import rms_agc  # grc-generated hier_block


class lilacsat1_ber_viterbi(gr.hier_block2):

    def __init__(self, bfo=12000, callsign='', ip='::', latitude=0, longitude=0, port=7355, recstart=''):
        gr.hier_block2.__init__(
            self, "LilacSat-1 decoder",
            gr.io_signature(1, 1, gr.sizeof_gr_complex*1),
            gr.io_signaturev(2, 2, [gr.sizeof_char*1, gr.sizeof_char*1]),
        )

        ##################################################
        # Parameters
        ##################################################
        self.bfo = bfo
        self.callsign = callsign
        self.ip = ip
        self.latitude = latitude
        self.longitude = longitude
        self.port = port
        self.recstart = recstart

        ##################################################
        # Variables
        ##################################################
        self.sps = sps = 5
        self.samp_per_sym = samp_per_sym = 5
        self.nfilts = nfilts = 16
        self.alpha = alpha = 0.35

        self.variable_constellation_0_0 = variable_constellation_0_0 = digital.constellation_calcdist(([-1, 1]), ([0, 1]), 2, 1).base()

        self.threshold = threshold = 3
        self.samp_rate = samp_rate = 48000
        self.rrc_taps_0 = rrc_taps_0 = firdes.root_raised_cosine(nfilts, nfilts, 1.0/float(samp_per_sym), 0.35, 11*samp_per_sym*nfilts)
        self.rrc_taps = rrc_taps = firdes.root_raised_cosine(nfilts, nfilts, 1.0/float(sps), alpha, 11*sps*nfilts)
        self.nfilts_0 = nfilts_0 = 16

        ##################################################
        # Blocks
        ##################################################
        self.rms_agc_0 = rms_agc(
            alpha=1e-2,
            reference=0.5,
        )
        self.low_pass_filter_0_0 = filter.fir_filter_ccf(1, firdes.low_pass(
        	1, samp_rate, 10000, 1000, window.WIN_HAMMING, 6.76))
        self.low_pass_filter_0 = filter.fir_filter_ccf(1, firdes.low_pass(
        	1, samp_rate, 6000, 500, window.WIN_HAMMING, 6.76))
        self.digital_pfb_clock_sync_xxx_0 = digital.pfb_clock_sync_ccf(sps, 0.05, (rrc_taps), nfilts, nfilts/2, 0.01, 1)
        self.digital_fll_band_edge_cc_0 = digital.fll_band_edge_cc(sps, 0.350, 100, 0.001)
        self.digital_costas_loop_cc_0_0 = digital.costas_loop_cc(0.1, 2, False)
        self.ccsds_viterbi_0_0 = ccsds_viterbi()
        self.ccsds_viterbi_0 = ccsds_viterbi()
        self.blocks_delay_0 = blocks.delay(gr.sizeof_float*1, 1)
        self.blocks_complex_to_real_0 = blocks.complex_to_real(1)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_complex_to_real_0, 0), (self.blocks_delay_0, 0))
        self.connect((self.blocks_complex_to_real_0, 0), (self.ccsds_viterbi_0, 0))
        self.connect((self.blocks_delay_0, 0), (self.ccsds_viterbi_0_0, 0))
        self.connect((self.ccsds_viterbi_0, 0), (self, 0))
        self.connect((self.ccsds_viterbi_0_0, 0), (self, 1))
        self.connect((self.digital_costas_loop_cc_0_0, 0), (self.blocks_complex_to_real_0, 0))
        self.connect((self.digital_fll_band_edge_cc_0, 0), (self.low_pass_filter_0, 0))
        self.connect((self.digital_pfb_clock_sync_xxx_0, 0), (self.digital_costas_loop_cc_0_0, 0))
        self.connect((self, 0), (self.low_pass_filter_0_0, 0))
        self.connect((self.low_pass_filter_0, 0), (self.digital_pfb_clock_sync_xxx_0, 0))
        self.connect((self.low_pass_filter_0_0, 0), (self.rms_agc_0, 0))
        self.connect((self.rms_agc_0, 0), (self.digital_fll_band_edge_cc_0, 0))

    def get_bfo(self):
        return self.bfo

    def set_bfo(self, bfo):
        self.bfo = bfo

    def get_callsign(self):
        return self.callsign

    def set_callsign(self, callsign):
        self.callsign = callsign

    def get_ip(self):
        return self.ip

    def set_ip(self, ip):
        self.ip = ip

    def get_latitude(self):
        return self.latitude

    def set_latitude(self, latitude):
        self.latitude = latitude

    def get_longitude(self):
        return self.longitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    def get_port(self):
        return self.port

    def set_port(self, port):
        self.port = port

    def get_recstart(self):
        return self.recstart

    def set_recstart(self, recstart):
        self.recstart = recstart

    def get_sps(self):
        return self.sps

    def set_sps(self, sps):
        self.sps = sps
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), self.alpha, 11*self.sps*self.nfilts))

    def get_samp_per_sym(self):
        return self.samp_per_sym

    def set_samp_per_sym(self, samp_per_sym):
        self.samp_per_sym = samp_per_sym
        self.set_rrc_taps_0(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.samp_per_sym), 0.35, 11*self.samp_per_sym*self.nfilts))

    def get_nfilts(self):
        return self.nfilts

    def set_nfilts(self, nfilts):
        self.nfilts = nfilts
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), self.alpha, 11*self.sps*self.nfilts))
        self.set_rrc_taps_0(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.samp_per_sym), 0.35, 11*self.samp_per_sym*self.nfilts))

    def get_alpha(self):
        return self.alpha

    def set_alpha(self, alpha):
        self.alpha = alpha
        self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), self.alpha, 11*self.sps*self.nfilts))

    def get_variable_constellation_0_0(self):
        return self.variable_constellation_0_0

    def set_variable_constellation_0_0(self, variable_constellation_0_0):
        self.variable_constellation_0_0 = variable_constellation_0_0

    def get_threshold(self):
        return self.threshold

    def set_threshold(self, threshold):
        self.threshold = threshold

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(1, self.samp_rate, 10000, 1000, window.WIN_HAMMING, 6.76))
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, self.samp_rate, 6000, 500, window.WIN_HAMMING, 6.76))

    def get_rrc_taps_0(self):
        return self.rrc_taps_0

    def set_rrc_taps_0(self, rrc_taps_0):
        self.rrc_taps_0 = rrc_taps_0

    def get_rrc_taps(self):
        return self.rrc_taps

    def set_rrc_taps(self, rrc_taps):
        self.rrc_taps = rrc_taps
        self.digital_pfb_clock_sync_xxx_0.update_taps((self.rrc_taps))

    def get_nfilts_0(self):
        return self.nfilts_0

    def set_nfilts_0(self, nfilts_0):
        self.nfilts_0 = nfilts_0



#!/usr/bin/env python3

# Copyright 2017, 2018, 2019, 2020 Daniel Estevez <daniel@destevez.net>
# Copyright 2022 The Regents of the University of Colorado
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from datetime import datetime
from construct import Adapter, BitsInteger, BitStruct, Container, Enum, \
                      Flag, GreedyBytes, If, Int8ub, Int16ub, Int32ub, \
                      Padding, RawCopy, Struct, Switch
from .ax25 import Header
from .cirbe_bct_soh import cirbe_bct_soh


PrimaryHeader = BitStruct(
    'ccsds_version' / BitsInteger(3),
    'packet_type' / Flag,
    'secondary_header_flag' / Flag,
    'is_stored_data' / Flag,
    'APID' / BitsInteger(10),
    'grouping_flag' / Enum(BitsInteger(2), GRP_MIDDLE=0, GRP_BEGIN=1,
                           GRP_END=2, GRP_FIRST_AND_LAST=3),
    'sequence_count' / BitsInteger(14),
    'packet_length' / BitsInteger(16)
)


SecondaryHeaderRaw = Struct(
    'time_stamp_seconds' / Int32ub,
    'sub_seconds' / Int8ub,
    Padding(1)
)


class TimeAdapter(Adapter):
    def _encode(self, obj, context, path=None):
        return Container()

    def _decode(self, obj, context, path=None):
        offset = datetime(2000, 1, 1, 12) - datetime(1970, 1, 1)
        return (datetime.utcfromtimestamp(obj.time_stamp_seconds) + offset)


SecondaryHeader = TimeAdapter(
    SecondaryHeaderRaw
)


cirbe_70cm = Struct(
    'ax25_header' / Header,
    'primary_header' / PrimaryHeader,
    'secondary_header' / If(
        lambda c: c.primary_header.secondary_header_flag,
        SecondaryHeader
    ),
    'packet' / Switch(
        lambda c: c.primary_header.APID,
        {
            0x050: cirbe_bct_soh
        }
    )
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import random

from gnuradio import gr, blocks, gr_unittest
import numpy as np

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import phase_unwrap


class qa_phase_unwrap(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_unwrap(self):
        data = np.cumsum(np.concatenate([
            np.full(2000, 0.1),
            np.full(2000, -0.1),
            np.full(3000, 0.25),
            np.full(3000, -0.2),
            ]))
        data_wrap = data % (2 * np.pi)
        # Do some arbitrary additions of multiples of 2 * pi
        data_wrap[::7] -= 2 * np.pi
        data_wrap[2::5] += 4 * np.pi

        self.source = blocks.vector_source_f(data_wrap, False)
        self.sink = blocks.vector_sink_b(12)
        self.unwrap = phase_unwrap()

        self.tb.connect(self.source, self.unwrap, self.sink)
        self.tb.run()

        data_out = np.array(self.sink.data(), 'uint8').reshape(-1, 12)
        int_cycles = data_out[:, :8].ravel().view('int64')
        frac_rad = data_out[:, 8:].ravel().view('float32')
        unwrapped = 2 * np.pi * int_cycles.astype('float') + frac_rad
        np.testing.assert_almost_equal(unwrapped, data, decimal=6)


if __name__ == '__main__':
    gr_unittest.run(qa_phase_unwrap)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import manchester_sync_cc, manchester_sync_ff


class qa_manchester_sync(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_manchester_sync_cc_no_offset(self):
        self.manchester_sync_test('c', False)

    def test_manchester_sync_cc_offset(self):
        self.manchester_sync_test('c', True)

    def test_manchester_sync_ff_no_offset(self):
        self.manchester_sync_test('f', False)

    def test_manchester_sync_ff_offset(self):
        self.manchester_sync_test('f', True)

    def manchester_sync_test(self, type_, offset):
        bits = 2 * np.random.randint(2, size=4096) - 1
        manchester_bits = np.repeat(bits, 2) * np.tile([1, -1], bits.size)
        if offset:
            manchester_bits[:-1] = manchester_bits[1:]

        source_block = {'f': blocks.vector_source_f,
                        'c': blocks.vector_source_c}[type_]
        self.source = source_block(manchester_bits)
        sink_block = {'f': blocks.vector_sink_f,
                      'c': blocks.vector_sink_c}[type_]
        self.sink = sink_block(1, bits.size)
        block_size = 32
        manchester_sync_block = {'f': manchester_sync_ff,
                                 'c': manchester_sync_cc}[type_]
        self.manchester_sync = manchester_sync_block(block_size)

        self.tb.connect(self.source, self.manchester_sync, self.sink)
        self.tb.run()

        sink_data = self.sink.data()
        print(type_, offset)
        if offset:
            np.testing.assert_equal(sink_data[1:], bits[1:])
        else:
            np.testing.assert_equal(sink_data, bits)


if __name__ == '__main__':
    gr_unittest.run(qa_manchester_sync)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import random

from gnuradio import gr, blocks, gr_unittest
import numpy as np

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import costas_loop_8apsk_cc


class qa_costas_loop_8apsk_cc(gr_unittest.TestCase):
    # This is inspired by the QA test of the costas loop
    # in gr-digital

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def setup_and_run_fg(self):
        self.source = blocks.vector_source_c(self.data, False)
        self.sink = blocks.vector_sink_c()

        self.tb.connect(self.source, self.costas, self.sink)
        self.tb.run()

    def test_zero_gain(self):
        loop_bw = 0.0
        self.costas = costas_loop_8apsk_cc(loop_bw)
        self.data = 100 * [complex(1, 0), ]
        self.setup_and_run_fg()
        self.assertComplexTuplesAlmostEqual(self.sink.data(), self.data, 5)

    def test_doesnt_diverge(self):
        # Test loop doesn't diverge given perfect data
        loop_bw = 0.1
        self.costas = costas_loop_8apsk_cc(loop_bw)
        self.data = [np.exp(1j * 2 * np.pi * random.randint(0, 7) / 7.0)
                     for _ in range(100)]
        self.setup_and_run_fg()
        self.assertComplexTuplesAlmostEqual(self.sink.data(), self.data, 5)

    def test_convergence(self):
        # Test convergence with a static rotation
        loop_bw = 0.1
        self.costas = costas_loop_8apsk_cc(loop_bw)

        rotation = np.exp(1j * 0.2)
        data = [np.exp(1j * 2 * np.pi * random.randint(0, 7) / 7.0)
                for _ in range(100)]
        self.data = [rotation * d for d in data]
        self.setup_and_run_fg()
        start = 50  # only check after loop has converged
        self.assertComplexTuplesAlmostEqual(
            self.sink.data()[start:], data[start:], 2)


if __name__ == '__main__':
    gr_unittest.run(qa_costas_loop_8apsk_cc)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020-2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy as np
from gnuradio import gr


class autopolarization(gr.sync_block):
    def __init__(self, fft_size=2048, fft_avg=1, iir_weight=0.1):
        gr.sync_block.__init__(
            self,
            name='autopolarization',
            in_sig=[np.complex64]*2,
            out_sig=[np.complex64]*2
        )
        self.fft_size = fft_size
        self.fft_avg = fft_avg
        self.set_output_multiple(fft_size * fft_avg)
        self.iir_weight = iir_weight
        self.noise_ampl = np.ones(2)
        self.sig_ampl = np.ones(2)
        self.phase = 0

    def work(self, input_items, output_items):
        x = np.array([inp.reshape((-1, self.fft_avg, self.fft_size))
                      for inp in input_items])
        # axes for x are:
        # (channel: 2, block, fft_num: fft_avg, fft_bin: fft_size)

        fx = np.fft.fft(x)
        fx_sq_avg = np.average(np.abs(fx)**2, axis=2)
        noise_ampl = np.sqrt(np.median(fx_sq_avg, axis=2))

        fx_sq_avg_ch_max = np.max(fx_sq_avg/noise_ampl[..., np.newaxis]**2,
                                  axis=0)
        peak_idx = np.argmax(fx_sq_avg_ch_max, axis=1)

        f_cross = np.average(fx[0] * np.conjugate(fx[1]), axis=1)
        phase = np.angle(f_cross[np.arange(f_cross.shape[0]),
                                 peak_idx])
        # technically this should have - noise_ampl
        sig_ampl = fx_sq_avg[:, np.arange(fx_sq_avg.shape[1]), peak_idx]
        sig_ampl = np.sqrt(np.clip(sig_ampl, 0, np.inf))

        # update IIR filters
        for j in range(x.shape[1]):
            noise_ampl[:, j] = (1-self.iir_weight)*self.noise_ampl \
                + self.iir_weight*noise_ampl[:, j]
            self.noise_ampl = noise_ampl[:, j]
            sig_ampl[:, j] = (1-self.iir_weight)*self.sig_ampl \
                + self.iir_weight*sig_ampl[:, j]
            self.sig_ampl = sig_ampl[:, j]
            phase_diff = phase[j] - self.phase
            phase_diff = (phase_diff + np.pi) % (2*np.pi) - np.pi
            phase[j] = self.phase + self.iir_weight*phase_diff
            self.phase = (phase[j] + np.pi) % (2*np.pi) - np.pi

        tau = np.log10(sig_ampl[0]/noise_ampl[0]+1e-6) \
            - np.log10(sig_ampl[1]/noise_ampl[1]+1e-6)
        tau = 20*tau/6*0.5 + 0.5
        tau = np.clip(tau, 0, 1)
        # note tau = 1 if sig[0] is 6dB stronger than sig[1] and
        # tau = 0 if sig[0] is 6dB weaker than sig[1]
        a_phase = (tau - 1) * phase
        b_phase = tau * phase
        # note -a_phase + b_phase = phase

        alpha = (np.exp(1j*a_phase)
                 * sig_ampl[0] / noise_ampl[0])[:, np.newaxis]
        beta = (np.exp(1j*b_phase)
                * sig_ampl[1] / noise_ampl[1])[:, np.newaxis]

        y = x.reshape((x.shape[0], x.shape[1], self.fft_avg * self.fft_size))
        y = y / noise_ampl[..., np.newaxis] * np.sqrt(self.fft_size)
        output_items[0][:] = (alpha * y[0] + beta * y[1]).ravel()
        output_items[1][:] = (
            np.conjugate(beta) * y[0] - np.conjugate(alpha) * y[1]).ravel()
        return len(output_items[0])



#
# Copyright 2004,2009,2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#

"""Misc utilities used at build time
"""

import re
import os
import os.path

from .build_utils_codes import *


# set srcdir to the directory that contains Makefile.am
try:
    srcdir = os.environ['srcdir']
except KeyError as e:
    srcdir = "."
srcdir = srcdir + '/'

# set do_makefile to either true or false dependeing on the environment
try:
    if os.environ['do_makefile'] == '0':
        do_makefile = False
    else:
        do_makefile = True
except KeyError as e:
    do_makefile = False

# set do_sources to either true or false dependeing on the environment
try:
    if os.environ['do_sources'] == '0':
        do_sources = False
    else:
        do_sources = True
except KeyError as e:
    do_sources = True


name_dict = {}


def log_output_name(name):
    (base, ext) = os.path.splitext(name)
    ext = ext[1:]                       # drop the leading '.'

    entry = name_dict.setdefault(ext, [])
    entry.append(name)


def open_and_log_name(name, dir):
    global do_sources
    if do_sources:
        f = open(name, dir)
    else:
        f = None
    log_output_name(name)
    return f


def expand_template(d, template_filename, extra=''):
    """Given a dictionary D and a TEMPLATE_FILENAME,
expand template into output file"""
    global do_sources
    output_extension = extract_extension(template_filename)
    template = open_src(template_filename, 'r')
    output_name = d['NAME'] + extra + '.' + output_extension
    log_output_name(output_name)
    if do_sources:
        output = open(output_name, 'w')
        do_substitution(d, template, output)
        output.close()
    template.close()


def output_glue(dirname):
    output_makefile_fragment()
    output_ifile_include(dirname)


def output_makefile_fragment():
    global do_makefile
    if not do_makefile:
        return
    # Overwrite the source, which must be writable. This should have been
    # checked for beforehand in the top-level Makefile.gen.gen .
    f = open(
        os.path.join(
            os.environ.get('gendir',
                           os.environ.get('srcdir', '.')),
            'Makefile.gen'), 'w')
    f.write('#\n# This file is machine generated. '
            ' All edits will be overwritten\n#\n')
    output_subfrag(f, 'h')
    output_subfrag(f, 'i')
    output_subfrag(f, 'cc')
    f.close()


def output_ifile_include(dirname):
    global do_sources
    if do_sources:
        f = open('%s_generated.i' % (dirname,), 'w')
        f.write('//\n// This file is machine generated.  '
                'All edits will be overwritten\n//\n')
        files = name_dict.setdefault('i', [])
        files.sort()
        f.write('%{\n')
        for file in files:
            f.write('#include <%s>\n' % (file[0:-1] + 'h',))
        f.write('%}\n\n')
        for file in files:
            f.write('%%include <%s>\n' % (file,))


def output_subfrag(f, ext):
    files = name_dict.setdefault(ext, [])
    files.sort()
    f.write('GENERATED_%s =' % ext.upper())
    for file in files:
        f.write(' \\\n\t%s' % (file,))
    f.write('\n\n')


def extract_extension(template_name):
    # template name is something like: GrFIRfilterXXX.h.t
    # we return everything between the penultimate . and .t
    mo = re.search(r'\.([a-z]+)\.t$', template_name)
    if not mo:
        raise ValueError(
            "Incorrectly formed template_name '%s'" % (template_name,))
    return mo.group(1)


def open_src(name, mode):
    global srcdir
    return open(os.path.join(srcdir, name), mode)


def do_substitution(d, in_file, out_file):
    def repl(match_obj):
        key = match_obj.group(1)
        # print key
        return d[key]

    inp = in_file.read()
    out = re.sub(r'@([a-zA-Z0-9_]+)@', repl, inp)
    out_file.write(out)


copyright = '''/* -*- c++ -*- */
/*
 * Copyright 2003,2004 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */
'''


def is_complex(code3):
    if i_code(code3) == 'c' or o_code(code3) == 'c':
        return '1'
    else:
        return '0'


def standard_dict(name, code3, package='gr'):
    d = {}
    d['NAME'] = name
    d['NAME_IMPL'] = name+'_impl'
    d['GUARD_NAME'] = 'INCLUDED_%s_%s_H' % (package.upper(), name.upper())
    d['GUARD_NAME_IMPL'] = (
        'INCLUDED_%s_%s_IMPL_H' % (package.upper(), name.upper()))
    d['BASE_NAME'] = re.sub('^' + package + '_', '', name)
    d['SPTR_NAME'] = '%s_sptr' % name
    d['WARNING'] = (
        'WARNING: this file is machine generated. Edits will be overwritten')
    d['COPYRIGHT'] = copyright
    d['TYPE'] = i_type(code3)
    d['I_TYPE'] = i_type(code3)
    d['O_TYPE'] = o_type(code3)
    d['TAP_TYPE'] = tap_type(code3)
    d['IS_COMPLEX'] = is_complex(code3)
    return d


def standard_dict2(name, code3, package):
    d = {}
    d['NAME'] = name
    d['BASE_NAME'] = name
    d['GUARD_NAME'] = 'INCLUDED_%s_%s_H' % (package.upper(), name.upper())
    d['WARNING'] = (
        'WARNING: this file is machine generated. Edits will be overwritten')
    d['COPYRIGHT'] = copyright
    d['TYPE'] = i_type(code3)
    d['I_TYPE'] = i_type(code3)
    d['O_TYPE'] = o_type(code3)
    d['TAP_TYPE'] = tap_type(code3)
    d['IS_COMPLEX'] = is_complex(code3)
    return d


def standard_impl_dict2(name, code3, package):
    d = {}
    d['NAME'] = name
    d['IMPL_NAME'] = name
    d['BASE_NAME'] = name.rstrip("impl").rstrip("_")
    d['GUARD_NAME'] = 'INCLUDED_%s_%s_H' % (package.upper(), name.upper())
    d['WARNING'] = (
        'WARNING: this file is machine generated. Edits will be overwritten')
    d['COPYRIGHT'] = copyright
    d['FIR_TYPE'] = "fir_filter_" + code3
    d['CFIR_TYPE'] = "fir_filter_" + code3[0:2] + 'c'
    d['TYPE'] = i_type(code3)
    d['I_TYPE'] = i_type(code3)
    d['O_TYPE'] = o_type(code3)
    d['TAP_TYPE'] = tap_type(code3)
    d['IS_COMPLEX'] = is_complex(code3)
    return d



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020,2023 Daniel Estevez <daniel@destevez.net>.
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy as np
import matplotlib.pyplot as plt
import scipy.signal
import scipy.special
from gnuradio import gr, digital, analog, blocks, fft, channels
from gnuradio.fft import window
from satellites.components.demodulators import (
    bpsk_demodulator, fsk_demodulator,
)


RAND_SEED = 42


class BERSim(gr.top_block):
    def __init__(self, ber_block, ncorrelations, lfsr_bits=16):
        gr.top_block.__init__(self)

        prn_len = 2**lfsr_bits
        prn = np.concatenate((scipy.signal.max_len_seq(lfsr_bits)[0], [1]))
        prn_fft_conj = np.conjugate(np.fft.fft(2*prn-1))

        self.source = blocks.vector_source_b(prn, True)

        self.ber_block = ber_block

        self.char2float = blocks.char_to_float(1, 0.5)
        self.add_const = blocks.add_const_ff(-1.0)
        self.source_vector = blocks.stream_to_vector(gr.sizeof_float, prn_len)

        self.fft = fft.fft_vfc(prn_len, True, [])
        self.prn_fft_source = blocks.vector_source_c(
            prn_fft_conj, True, prn_len)
        self.multiply_ffts = blocks.multiply_vcc(prn_len)
        self.ifft = fft.fft_vcc(prn_len, False, np.ones(prn_len)/prn_len**2)
        self.corr_mag = blocks.complex_to_mag(prn_len)
        self.max_corr = blocks.max_ff(prn_len, 1)
        self.multiply_const = blocks.multiply_const_ff(-0.5)
        self.add_const2 = blocks.add_const_ff(0.5)

        self.head = blocks.head(gr.sizeof_float, ncorrelations)
        self.sink = blocks.vector_sink_f()

        self.connect(
            self.source, self.ber_block, self.char2float, self.add_const,
            self.source_vector, self.fft, (self.multiply_ffts, 0),
            self.ifft, self.corr_mag, self.max_corr, self.multiply_const,
            self.add_const2, self.head, self.sink)
        self.connect(self.prn_fft_source, (self.multiply_ffts, 1))


class BPSK(gr.hier_block2):
    def __init__(self, ebn0, nbits):
        gr.hier_block2.__init__(self, 'BPSK',
                                gr.io_signature(1, 1, gr.sizeof_char),
                                gr.io_signature(1, 1, gr.sizeof_char))

        samp_rate = 48e3
        sps = 5
        self.head = blocks.head(gr.sizeof_char, nbits)
        self.pack = blocks.pack_k_bits_bb(8)
        self.bpsk_constellation = digital.constellation_bpsk().base()
        self.modulator = digital.generic_mod(
            constellation=self.bpsk_constellation,
            differential=False,
            samples_per_symbol=sps,
            pre_diff_code=True,
            excess_bw=0.35,
            verbose=False,
            log=False)

        spb = sps
        self.channel = channels.channel_model(
            np.sqrt(spb)/10**(ebn0/20), 0, 1.0, [1], RAND_SEED, False)

        self.demod = bpsk_demodulator(samp_rate/sps, samp_rate, iq=True)
        self.slice = digital.binary_slicer_fb()

        self.connect(self, self.head, self.pack, self.modulator, self.channel,
                     self.demod, self.slice, self)


class FSK(gr.hier_block2):
    def __init__(self, ebn0, nbits):
        gr.hier_block2.__init__(self, 'FSK',
                                gr.io_signature(1, 1, gr.sizeof_char),
                                gr.io_signature(1, 1, gr.sizeof_char))

        samp_rate = 48e3
        sps = 5
        deviation = 5000
        bt = 1.0
        self.head = blocks.head(gr.sizeof_char, nbits)
        self.pack = blocks.pack_k_bits_bb(8)
        self.modulator = digital.gfsk_mod(
            samples_per_symbol=sps,
            sensitivity=2*np.pi*deviation/samp_rate,
            bt=bt,
            verbose=False,
            log=False)

        spb = sps
        self.channel = channels.channel_model(
            np.sqrt(spb)/10**(ebn0/20), 0, 1.0, [1], RAND_SEED, False)

        self.demod = fsk_demodulator(
            samp_rate/sps, samp_rate, deviation=deviation, iq=True)
        self.slice = digital.binary_slicer_fb()

        self.connect(self, self.head, self.pack, self.modulator, self.channel,
                     self.demod, self.slice, self)


def compute_ber(ber_block_class, ebn0, drop_correlations=2):
    ncorrelations = 100
    nbits = int(ncorrelations * 2**16 * 1.1)  # 1.1 to add a bit of margin
    fg = BERSim(ber_block_class(ebn0, nbits), ncorrelations)
    print(f'Computing BPSK BER for EbN0 = {ebn0:.01f} dB')
    fg.run()
    return np.average(fg.sink.data()[drop_correlations:])


def ber_bpsk_awgn(ebn0):
    """Calculates theoretical bit error rate in AWGN

    (for BPSK and given Eb/N0)"""
    return 0.5 * scipy.special.erfc(10**(ebn0/20))


def ber_fsk_awgn(ebn0):
    """Calculates theoretical bit error rate in AWGN

    (for FPSK and given Eb/N0)"""
    return 0.5 * np.exp(-0.5*10**(ebn0/10))


if __name__ == '__main__':
    print('Computing BPSK BER')
    ebn0s = np.arange(-2, 10.5, 0.5)
    bers = [compute_ber(BPSK, ebn0) for ebn0 in ebn0s]
    bers_theory = [ber_bpsk_awgn(ebn0) for ebn0 in ebn0s]

    fig, ax = plt.subplots()
    ax.semilogy(ebn0s, bers_theory, '.-', label='Theoretical BPSK BER')
    ax.semilogy(ebn0s, bers, '.-', label='gr-satellites demodulator')
    ax.set_title('BPSK BER')
    ax.set_xlabel('Eb/N0 (dB)')
    ax.set_ylabel('BER')
    ax.legend()
    ax.grid()
    fig.savefig('ber_bpsk.png')
    print('Saved output to ber_bpsk.png')

    print('Computing FSK BER')
    ebn0s = np.arange(-2, 14.5, 0.5)
    bers = [compute_ber(FSK, ebn0) for ebn0 in ebn0s]
    bers_theory = [ber_fsk_awgn(ebn0) for ebn0 in ebn0s]

    fig, ax = plt.subplots()
    ax.semilogy(ebn0s, bers_theory, '.-',
                label='Theoretical non-coherent FSK BER')
    ax.semilogy(ebn0s, bers, '.-', label='gr-satellites demodulator')
    ax.set_title('FSK BER deviation 5kHz, GFSK BT = 1')
    ax.set_xlabel('Eb/N0 (dB)')
    ax.set_ylabel('BER')
    ax.legend()
    ax.grid()
    fig.savefig('ber_fsk.png')
    print('Saved output to ber_fsk.png')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import nrzi_encode, nrzi_decode


class qa_nrzi(gr_unittest.TestCase):
    def setUp(self):
        test_size = 256
        self.data = np.random.randint(0, 2, test_size, dtype='uint8')
        self.source = blocks.vector_source_b(self.data, False, 1, [])
        self.sink = blocks.vector_sink_b(1, 0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None
        del self.data
        del self.source
        del self.sink

    def test_encode(self):
        """Performs NRZI encode and checks the result"""
        encode = nrzi_encode()

        self.tb.connect(self.source, encode, self.sink)
        self.tb.start()
        self.tb.wait()

        expected = np.cumsum((1 ^ self.data) & 1) & 1

        np.testing.assert_equal(
            self.sink.data(), expected,
            'NRZI encode output does not match expected result')

    def test_decode(self):
        """Performs NRZI decode and checks the result"""
        decode = nrzi_decode()

        self.tb.connect(self.source, decode, self.sink)
        self.tb.start()
        self.tb.wait()

        expected = self.data[1:] ^ self.data[:-1] ^ 1

        np.testing.assert_equal(
            self.sink.data()[1:], expected,
            'NRZI decode output does not match expected result')

    def test_encode_decode(self):
        """Performs NRZI encode and decode and checks the result"""
        encode = nrzi_encode()
        decode = nrzi_decode()

        self.tb.connect(self.source, encode, decode, self.sink)
        self.tb.start()
        self.tb.wait()

        np.testing.assert_equal(
            self.sink.data(), self.data,
            'NRZI encoded and decoded output does not match input')


if __name__ == '__main__':
    gr_unittest.run(qa_nrzi)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json

from gnuradio import gr
import pmt
import websocket


class bme_ws_submitter(gr.basic_block):
    """
    Submits telemetry to wss://gnd.bme.hu:8070/send
    """
    def __init__(self):
        gr.basic_block.__init__(
            self,
            name='bme_ws_submitter',
            in_sig=[],
            out_sig=[])

        self.ws = websocket.WebSocket()
        url = 'wss://gnd.bme.hu:8070/send'
        try:
            self.ws.connect(url)
        except Exception:
            print(f'could not connect to {url}; '
                  'disabling telemetry submission')
            self.ws = None

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)

    def __del__(self):
        if self.ws is not None:
            self.ws.close()

    def submit(self, frame):
        if self.ws is None:
            return
        data = f'"data": "{frame.hex().upper()}"'
        self.ws.send(data)
        response = self.ws.recv()
        response = json.loads('{' + response + '}')
        error = any([v != 0 for k, v in response.items()
                     if 'error' in k])
        if error:
            print(f'server did not accept frame; returned: {response}')

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print('[ERROR] Received invalid message type. Expected u8vector')
            return

        frame = bytes(pmt.u8vector_elements(msg))
        try:
            self.submit(frame)
        except Exception as e:
            print(f'failed to submit frame: {e}')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import pdu_add_meta


class qa_pdu_add_meta(gr_unittest.TestCase):
    def test_pdu_add_meta(self):
        tb = gr.top_block()
        dbg = blocks.message_debug()
        meta = pmt.make_dict()
        meta = pmt.dict_add(meta, pmt.intern('k1'), pmt.intern('v1'))
        meta = pmt.dict_add(meta, pmt.intern('k2'), pmt.intern('v2'))
        add_meta = pdu_add_meta(meta)

        pdu = pmt.cons(pmt.PMT_NIL, pmt.make_u8vector(10, 0))

        tb.msg_connect((add_meta, 'out'), (dbg, 'store'))

        add_meta.to_basic_block()._post(pmt.intern('in'), pdu)
        add_meta.to_basic_block()._post(
            pmt.intern('system'),
            pmt.cons(pmt.intern('done'), pmt.from_long(1)))

        tb.start()
        tb.wait()

        pdu_out = dbg.get_message(0)
        meta_out = pmt.car(pdu_out)
        self.assertTrue(pmt.dict_has_key(meta_out, pmt.intern('k1')),
                        'Test key k1 not in output PDU metadata')
        self.assertTrue(pmt.dict_has_key(meta_out, pmt.intern('k2')),
                        'Test key k1 not in output PDU metadata')
        self.assertEqual(pmt.u8vector_elements(pmt.cdr(pdu_out)),
                         pmt.u8vector_elements(pmt.cdr(pdu)),
                         'Output PDU data does not match input PDU data')


if __name__ == '__main__':
    gr_unittest.run(qa_pdu_add_meta)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020, 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt


# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites.components.datasinks import kiss_server_sink


class qa_kiss_server_sink(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_instance(self):
        """Tries to create a KISS server sink instance

        It uses several combinations of parameters
        """
        kiss_server_sink('', 1234)
        kiss_server_sink('127.0.0.1', 4567)


if __name__ == '__main__':
    gr_unittest.run(qa_kiss_server_sink)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2021-2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, gr_unittest
import numpy as np
import pmt

# bootstrap satellites module, even from build dir
try:
    import python as satellites
except ImportError:
    pass
else:
    import sys
    sys.modules['satellites'] = satellites

from satellites import fixedlen_to_pdu
from satellites.grtypes import byte_t


class qa_fixedlen_tagger(gr_unittest.TestCase):
    def setUp(self):
        self.syncword_tag = 'syncword'
        self.packetlen_tag = 'packet_len'
        self.packet_len = 100
        self.data = np.arange(3000, dtype='uint8')
        self.tag_positions = [50, 213, 217, 230, 1530, 1531]
        tags = [gr.python_to_tag((j, pmt.intern(self.syncword_tag),
                                  pmt.intern('sync'), pmt.intern('test_src')))
                for j in self.tag_positions]
        self.source = blocks.vector_source_b(self.data, False, 1, tags)
        self.debug = blocks.message_debug()
        self.fixedlen_to_pdu = fixedlen_to_pdu(
            byte_t, self.syncword_tag, self.packet_len)
        self.tb = gr.top_block()
        self.tb.connect(self.source, self.fixedlen_to_pdu)
        self.tb.msg_connect((self.fixedlen_to_pdu, 'pdus'),
                            (self.debug, 'store'))

    def tearDown(self):
        self.tb = None

    def test_tagger(self):
        """Runs some data through Fixedlen to PDU and checks resulting PDUs"""
        self.tb.start()
        self.tb.wait()

        self.assertEqual(self.debug.num_messages(), len(self.tag_positions),
                         'Unexpected number of PDUs')
        for j, pos in enumerate(self.tag_positions):
            pdu = np.array(pmt.u8vector_elements(
                pmt.cdr(self.debug.get_message(j))), dtype='uint8')
            expected = self.data[pos:pos+self.packet_len]
            np.testing.assert_equal(pdu, expected,
                                    'PDU values do not match expected')


if __name__ == '__main__':
    gr_unittest.run(qa_fixedlen_tagger)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from . import crc_check


def crc16_arc(swap_endianness=True, discard_crc=True):
    return crc_check(16, 0x8005, 0x0, 0x0, True, True,
                     swap_endianness, discard_crc)


def crc16_ccitt_x25(swap_endianness=True, discard_crc=True):
    return crc_check(16, 0x1021, 0xFFFF, 0xFFFF, True, True,
                     swap_endianness, discard_crc)


def crc16_ccitt_false(swap_endianness=False, discard_crc=True):
    return crc_check(16, 0x1021, 0xFFFF, 0x0, False, False,
                     swap_endianness, discard_crc)


def crc16_ccitt_zero(swap_endianness=False, discard_crc=True):
    return crc_check(16, 0x1021, 0x0, 0x0, False, False,
                     swap_endianness, discard_crc)


def crc16_cc11xx(discard_crc=True):
    return crc_check(16, 0x8005, 0xFFFF, 0x0, False, False,
                     False, discard_crc)



from ..adapters import LinearAdapter, PolynomialAdapter
from construct import Adapter, BitsInteger, BitStruct, Container, Enum, Flag, \
                      Float64b, Int8ub, Int16ub, Int32ub, Padding, Struct, \
                      Switch


soh_l0 = BitStruct(
    'wdt_2sec_cnt' / LinearAdapter(1.0, BitsInteger(3)),
    'reset_armed' / Enum(BitsInteger(1), ARMED=1, NOT_ARMED=0),
    'wdt_stat' / Enum(BitsInteger(1), NO_WDT=0, WDT=1),
    'wdt_en' / Enum(BitsInteger(1), DISABLED=0, ENABLED=1),
    'table_select' / Enum(BitsInteger(1), FLASH=0, COMPILED=1),
    'boot_relay' / Enum(BitsInteger(1), PRIMARY=1, REDUNDANT=0),
    'l0_acpt_cnt' / LinearAdapter(1.0, BitsInteger(8)),
    'l0_rjct_cnt' / LinearAdapter(1.0, BitsInteger(8)),
    'hw_sec_cnt' / LinearAdapter(1.0, BitsInteger(8)),
    Padding(64),
    'time_tag' / LinearAdapter(1.0, BitsInteger(32)),
    Padding(80),
    'spare_end' / LinearAdapter(1.0, BitsInteger(48)),
)


soh_command_tlm = BitStruct(
    'cmd_status' / Enum(BitsInteger(8), OK=0, BAD_APID=1, BAD_OPCODE=2,
                        BAD_DATA=3, NOW_READING=4, DONE_READING=5, IDLE=6,
                        NO_CMD_DATA=7, CMD_SRVC_OVERRUN=8, CMD_APID_OVERRUN=9,
                        INCORRECT_WHEEL_MODE=10, BAD_ELEMENT=11,
                        TABLES_BUSY=12, FLASH_NOT_ARMED=13,
                        THRUSTERS_DISABLED=14, ATT_ERR_TOO_HIGH=15,
                        ASYNC_REFUSED=16, DRIVER_ERROR=17),
    'realtime_cmd_accept_count' / LinearAdapter(1.0, BitsInteger(8)),
    'realtime_cmd_reject_count' / LinearAdapter(1.0, BitsInteger(8)),
    'stored_cmd_accept_count' / LinearAdapter(1.0, BitsInteger(8)),
    'stored_cmd_reject_count' / LinearAdapter(1.0, BitsInteger(8)),
)


soh_general = BitStruct(
    'scrub_status_overall' / Enum(BitsInteger(8), OK=0, FAIL=-1, IN_PROG=1,
                                  ABORTED=-2),
    'image_booted' / Enum(BitsInteger(8), PRIMARY=0, REDUNDANT=1),
    'image_auto_failover' / Enum(BitsInteger(8), OK=0, FAIL=1),
    'inertia_index' / LinearAdapter(1.0, BitsInteger(8)),
)


soh_time = Struct(
    'tai_seconds' / Float64b,
    'time_valid' / Enum(Int8ub, YES=1, NO=0),
    'rtc_health1_pack' / BitStruct(
        'health1_pack_spare2' / LinearAdapter(1.0, BitsInteger(1)),
        'rtc_osc_rst_count' / LinearAdapter(1.0, BitsInteger(3)),
        'rtc_init_time_at_boot' / Enum(BitsInteger(1), NO=0, YES=1),
        'rtc_sync_stat' / Enum(BitsInteger(1), OFF=0, ON=1),
        'rtc_alive' / Enum(BitsInteger(1), NO=0, YES=1),
        'rtc_power' / Enum(BitsInteger(1), OFF=0, ON=1),)
)


soh_refs = BitStruct(
    'position_wrt_eci1' / LinearAdapter(50000.002500000126, BitsInteger(32)),
    'position_wrt_eci2' / LinearAdapter(50000.002500000126, BitsInteger(32)),
    'position_wrt_eci3' / LinearAdapter(50000.002500000126, BitsInteger(32)),
    'velocity_wrt_eci1' / LinearAdapter(200000000.0, BitsInteger(32)),
    'velocity_wrt_eci2' / LinearAdapter(200000000.0, BitsInteger(32)),
    'velocity_wrt_eci3' / LinearAdapter(200000000.0, BitsInteger(32)),
    'modeled_sun_vector_body1' / LinearAdapter(25000.000625000015,
                                               BitsInteger(16)),
    'modeled_sun_vector_body2' / LinearAdapter(25000.000625000015,
                                               BitsInteger(16)),
    'modeled_sun_vector_body3' / LinearAdapter(25000.000625000015,
                                               BitsInteger(16)),
    'mag_model_vector_body1' / LinearAdapter(200000000.0, BitsInteger(16)),
    'mag_model_vector_body2' / LinearAdapter(200000000.0, BitsInteger(16)),
    'mag_model_vector_body3' / LinearAdapter(200000000.0, BitsInteger(16)),
    'refs_valid' / Enum(BitsInteger(8), YES=1, NO=0),
    'run_low_rate_task' / Enum(BitsInteger(8), YES=1, NO=0),
)


soh_att_det = BitStruct(
    'q_body_wrt_eci1' / LinearAdapter(2000000040.000001, BitsInteger(32)),
    'q_body_wrt_eci2' / LinearAdapter(2000000040.000001, BitsInteger(32)),
    'q_body_wrt_eci3' / LinearAdapter(2000000040.000001, BitsInteger(32)),
    'q_body_wrt_eci4' / LinearAdapter(2000000040.000001, BitsInteger(32)),
    'tracker_sol_mixed' / Enum(BitsInteger(1), NO=0, YES=1),
    Padding(1),
    'tracker2_data_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'tracker1_data_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'imu_data_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'meas_rate_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'meas_att_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'attitude_valid' / Enum(BitsInteger(1), NO=0, YES=1),
)


soh_att_cmd = BitStruct(
    'hr_cycle_safe_mode' / LinearAdapter(1.0, BitsInteger(32)),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(1)),
    'sun_point_reason' / Enum(BitsInteger(3), BOOT=0, COMMAND=1,
                              ATTITUDE_INVALID=2, TIME_INVALID=3,
                              REFS_INVALID=4),
    'recommend_sun_point' / Enum(BitsInteger(1), NO=0, YES=1),
    Padding(2),
    'adcs_mode' / Enum(BitsInteger(1), SUN_POINT=0, FINE_REF_POINT=1),
)


soh_rw_drive = BitStruct(
    'filtered_speed_rpm1' / LinearAdapter(2.4999999375000015, BitsInteger(16)),
    'filtered_speed_rpm2' / LinearAdapter(2.4999999375000015, BitsInteger(16)),
    'filtered_speed_rpm3' / LinearAdapter(2.4999999375000015, BitsInteger(16)),
)


soh_tracker = BitStruct(
    'operating_mode' / Enum(BitsInteger(8), IDLE=0, INITIALIZE=1, STARID=2,
                            TRACK=3, PHOTO=4, CAL=5, BLOCK=6),
    'star_id_step' / Enum(BitsInteger(8), IDLE=0, INITIALIZE=1,
                          WAITING_FOR_IMAGE1=2, WAITING_FOR_IMAGE2=3,
                          CALCULATE_RATE=4, MAKE_UNIT_VECTORS=5,
                          AWAITING_TRISTAR=6, OK_FOUND_4=7, OK_FOUND_3=8,
                          TIME_OUT=9, SPARE=10, NO_MATCH=11),
    'att_status' / Enum(BitsInteger(8), OK=0, PENDING=1, BAD=2,
                        TOO_FEW_STARS=3, QUEST_FAILED=4, RESIDUALS_TOO_HIGH=5,
                        TOO_CLOSE_TO_EDGE=6, PIX_AMP_TOO_LOW=7,
                        PIX_AMP_TOO_HIGH=8, BACKGND_TOO_HIGH=9,
                        TRACK_FAILURE=10, PIX_SUM_TOO_LOW=11, UNUSED=12,
                        TOO_DIM_FOR_STARID=13, TOO_MANY_GROUPS=14,
                        TOO_FEW_GROUPS=15, CHANNEL_DISABLED=16,
                        TRACK_BLK_OVERLAP=17, OK_FOR_STARID=18,
                        TOO_CLOSE_TO_OTHER=19, TOO_MANY_PIXELS=20,
                        TOO_MANY_COLUMNS=21, TOO_MANY_ROWS=22, OPEN=23,
                        CLOSED=24, RATE_TOO_HIGH=25),
    'num_attitude_stars' / LinearAdapter(1.0, BitsInteger(8)),
)


soh_att_ctrl = BitStruct(
    'eigen_error' / LinearAdapter(659999963.0400021, BitsInteger(32)),
    'sun_point_angle_error' / LinearAdapter(333.33335555555703,
                                            BitsInteger(16)),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(2)),
    'sun_source_failover' / Enum(BitsInteger(1), OK=0, FAIL=1),
    'sun_avoid_flag' / Enum(BitsInteger(1), NO=0, YES=1),
    Padding(1),
    'on_sun_flag' / Enum(BitsInteger(1), NO=0, YES=1),
    'momentum_too_high' / Enum(BitsInteger(1), NO=0, YES=1),
    'att_ctrl_active' / Enum(BitsInteger(1), NO=0, YES=1),
)


soh_momentum = BitStruct(
    'total_momentum_mag' / LinearAdapter(1999.9999200000032, BitsInteger(16)),
    'duty_cycle1' / LinearAdapter(1.0, BitsInteger(8)),
    'duty_cycle2' / LinearAdapter(1.0, BitsInteger(8)),
    'duty_cycle3' / LinearAdapter(1.0, BitsInteger(8)),
    'torque_rod_mode1' / Enum(BitsInteger(8), OFF=0, ON_POS=1,
                              ON_NEG=2, AUTO=3, MEASURED=4, MODELED=5,
                              DELAYED_AUTO=6, NO_FIELD_VALID=7, UNUSED1=8,
                              UNUSED2=9, BIDIRECTIONAL=10, POS_ONLY=11,
                              NEG_ONLY=12),
    'torque_rod_mode2' / Enum(BitsInteger(8), OFF=0, ON_POS=1,
                              ON_NEG=2, AUTO=3, MEASURED=4, MODELED=5,
                              DELAYED_AUTO=6, NO_FIELD_VALID=7, UNUSED1=8,
                              UNUSED2=9, BIDIRECTIONAL=10, POS_ONLY=11,
                              NEG_ONLY=12),
    'torque_rod_mode3' / Enum(BitsInteger(8), OFF=0, ON_POS=1,
                              ON_NEG=2, AUTO=3, MEASURED=4, MODELED=5,
                              DELAYED_AUTO=6, NO_FIELD_VALID=7, UNUSED1=8,
                              UNUSED2=9, BIDIRECTIONAL=10, POS_ONLY=11,
                              NEG_ONLY=12),
    'torque_rod_firing_pack_spare' / LinearAdapter(1.0, BitsInteger(1)),
    'torque_rod_direction3' / Enum(BitsInteger(1), POS=0, NEG=1),
    'torque_rod_direction2' / Enum(BitsInteger(1), POS=0, NEG=1),
    'torque_rod_direction1' / Enum(BitsInteger(1), POS=0, NEG=1),
    Padding(1),
    'torque_rod_enable3' / Enum(BitsInteger(1), DS=0, EN=1),
    'torque_rod_enable2' / Enum(BitsInteger(1), DS=0, EN=1),
    'torque_rod_enable1' / Enum(BitsInteger(1), DS=0, EN=1),
    'health1_pack_spare2' / LinearAdapter(1.0, BitsInteger(2)),
    'mag_source_failover' / Enum(BitsInteger(1), OK=0, FAULT=1),
    'tr_fault' / Enum(BitsInteger(1), FAULT=0, OK=1),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(1)),
    'momentum_vector_enabled' / Enum(BitsInteger(1), NO=0, YES=1),
    'momentum_vector_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'tr_drive_power_state' / Enum(BitsInteger(1), OFF=0, ON=1),
)

soh_css = BitStruct(
    'sun_vector_body1' / LinearAdapter(10000.000300000009, BitsInteger(16)),
    'sun_vector_body2' / LinearAdapter(10000.000300000009, BitsInteger(16)),
    'sun_vector_body3' / LinearAdapter(10000.000300000009, BitsInteger(16)),
    'sun_vector_status' / Enum(BitsInteger(8), GOOD=0, COARSE=1, BAD=2),
    'css_invalid_count' / LinearAdapter(1.0, BitsInteger(16)),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(1)),
    'sun_sensor_used' / LinearAdapter(1.0, BitsInteger(3)),
    'css_test_mode' / Enum(BitsInteger(1), NO=0, YES=1),
    'sun_vector_enabled' / Enum(BitsInteger(1), NO=0, YES=1),
    'meas_sun_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'css_power_state' / Enum(BitsInteger(1), OFF=0, ON=1),
)

soh_mag = BitStruct(
    'mag_vector_body1' / LinearAdapter(200000000.0, BitsInteger(16)),
    'mag_vector_body2' / LinearAdapter(200000000.0, BitsInteger(16)),
    'mag_vector_body3' / LinearAdapter(200000000.0, BitsInteger(16)),
    'mag_invalid_count' / LinearAdapter(1.0, BitsInteger(16)),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(1)),
    'mag_sensor_used' / LinearAdapter(1.0, BitsInteger(3)),
    'mag_test_mode' / Enum(BitsInteger(1), NO=0, YES=1),
    'mag_vector_enabled' / Enum(BitsInteger(1), NO=0, YES=1),
    'mag_vector_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'mag_power_state' / Enum(BitsInteger(1), OFF=0, ON=1),
)

soh_imu = BitStruct(
    'imu_avg_vector_body1' / LinearAdapter(100000.0030000001, BitsInteger(16)),
    'imu_avg_vector_body2' / LinearAdapter(100000.0030000001, BitsInteger(16)),
    'imu_avg_vector_body3' / LinearAdapter(100000.0030000001, BitsInteger(16)),
    'imu_invalid_count' / LinearAdapter(1.0, BitsInteger(16)),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(3)),
    'imu_valid_packets' / Enum(BitsInteger(1), NO=0, YES=1),
    'imu_test_mode' / Enum(BitsInteger(1), NO=0, YES=1),
    'imu_vector_enabled' / Enum(BitsInteger(1), NO=0, YES=1),
    'imu_vector_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'imu_power_state' / Enum(BitsInteger(1), OFF=0, ON=1),
)

soh_clock_sync = BitStruct(
    'hr_run_count' / LinearAdapter(1.0, BitsInteger(32)),
    'hr_exec_time_ms1' / LinearAdapter(1.0, BitsInteger(8)),
    'hr_exec_time_ms2' / LinearAdapter(1.0, BitsInteger(8)),
    'hr_exec_time_ms3' / LinearAdapter(1.0, BitsInteger(8)),
    'hr_exec_time_ms4' / LinearAdapter(1.0, BitsInteger(8)),
    'hr_exec_time_ms5' / LinearAdapter(1.0, BitsInteger(8)),
)

soh_analogs = BitStruct(
    'battery_voltage' / LinearAdapter(499.99997500000126, BitsInteger(16)),
)

soh_gps = BitStruct(
    'gps_cycles_since_crc_data' / LinearAdapter(1.0, BitsInteger(32)),
    'gps_lock_count' / LinearAdapter(1.0, BitsInteger(16)),
    'msg_used_satellites' / LinearAdapter(1.0, BitsInteger(8)),
    'gps_pos_lock' / Enum(BitsInteger(1), NO=0, YES=1),
    'gps_time_lock' / Enum(BitsInteger(1), NO=0, YES=1),
    'msg_data_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'gps_new_data_received' / Enum(BitsInteger(1), NO=0, YES=1),
    Padding(1),
    'gps_enable' / Enum(BitsInteger(1), NO=0, YES=1),
    'gps_valid' / Enum(BitsInteger(1), NO=0, YES=1),
    'health1_pack_spare1' / LinearAdapter(1.0, BitsInteger(1)),
)

soh_event_check = BitStruct(
    'latched_resp_fire_pack_bit8' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit7' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit6' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit5' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit4' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit3' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit2' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit1' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit16' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit15' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit14' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit13' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit12' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit11' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit10' / LinearAdapter(1.0, BitsInteger(1)),
    'latched_resp_fire_pack_bit9' / LinearAdapter(1.0, BitsInteger(1)),
)

soh_radio = BitStruct(
    'sd_minute_cur' / LinearAdapter(1.0, BitsInteger(32)),
    'sd_percent_used_total' / LinearAdapter(1.0, BitsInteger(8)),
    'sd_ok' / Enum(BitsInteger(8), YES=1, NO=0),
    'sd_fault_count' / LinearAdapter(1.0, BitsInteger(8)),
    'sq_channel' / LinearAdapter(1.0, BitsInteger(8)),
    'sq_trap_count' / LinearAdapter(1.0, BitsInteger(8)),
    'sq_temp' / LinearAdapter(1.0, BitsInteger(8)),
    'sdr_tx_tx_frames' / LinearAdapter(1.0, BitsInteger(32)),
    'sdr_tx_tx_power' / LinearAdapter(1.0, BitsInteger(8)),
    'sdr_tx_temp' / LinearAdapter(1.0, BitsInteger(8)),
    'sdr_tx_comm_error' / Enum(BitsInteger(8), NO=0, YES=1),
)

soh_tracker_ctrl = BitStruct(
    'tracker_att_valid' / Enum(BitsInteger(8), YES=1, NO=0)
)

cirbe_bct_soh = Struct(
    'soh_l0' / soh_l0,
    'soh_command_tlm' / soh_command_tlm,
    'soh_general' / soh_general,
    'soh_time' / soh_time,
    'soh_refs' / soh_refs,
    'soh_att_det' / soh_att_det,
    'soh_att_cmd' / soh_att_cmd,
    'soh_rw_drive' / soh_rw_drive,
    'soh_tracker' / soh_tracker,
    'soh_att_ctrl' / soh_att_ctrl,
    'soh_momentum' / soh_momentum,
    'soh_css' / soh_css,
    'soh_mag' / soh_mag,
    'soh_imu' / soh_imu,
    'soh_clock_sync' / soh_clock_sync,
    'soh_analogs' / soh_analogs,
    'soh_gps' / soh_gps,
    'soh_event_check' / soh_event_check,
    'soh_radio' / soh_radio,
    'soh_tracker_ctrl' / soh_tracker_ctrl,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pathlib import Path


def get_cfg(norad):
    """
    Opens and returns the local configuration for selected satellite
    """
    config_dir = Path.home() / '.gr_satellites'
    config_filename = config_dir / 'sat.cfg'

    if not config_filename.exists():
        return []

    with open(config_filename) as f:
        for row in f:
            sat = row.strip().split(' ')
            if len(sat) > 1 and norad == int(sat[0]):
                return sat[1:]
    return []



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr_unittest

import satyaml


class qa_satyaml(gr_unittest.TestCase):

    def test_check_all_satyaml_files(self):
        for yml in satyaml.yamlfiles.yaml_files():
            with self.subTest(satyaml=yml):
                satyaml.yamlfiles.check_yaml(yml)


if __name__ == '__main__':
    gr_unittest.run(qa_satyaml)
